-----------------------------------------------------------------
-- Header
-----------------------------------------------------------------
	do
		--Creates a new UI frame called "BreadCrumbs"
		BreadCrumbs = CreateFrame("Frame", "BreadCrumbs", UIParent);
		-- Sets the name of the addon
		BreadCrumbs.Name = "BreadCrumbs";
		-- Pulls back the title of the addon from the TOC file
		BreadCrumbs.Title = GetAddOnMetadata(BreadCrumbs.Name, "Title");
		-- Pulls back the current version of the file from the TOC file
		BreadCrumbs.Version = GetAddOnMetadata(BreadCrumbs.Name, "Version");
		-- Says what stage the addon loading is at.
		BreadCrumbs.LoadedStatus = {};
		-- Say that the addon has not loaded yet.
		BreadCrumbs.LoadedStatus["Initialized"] = 0;
		-- Specifies what level the addon is "Running"
		BreadCrumbs.LoadedStatus["RunLevel"] = 2;
	end;
	
	local minimapParent, lastX, lastY = Minimap, 0, 0;
-----------------------------------------------------------------

-----------------------------------------------------------------
-- Libraries
-----------------------------------------------------------------
	-- Registering local variables outside hidder do
	local ZasName, Msg, ErrMsg, DBugMsg, ErrDBugMsg, InitDBug, ZMC_COLOURS, ZMC_Settings, L, TableCount, AceConfigDialog, AceConfigRegistry, AceGUI, Astrolabe, AceHook, AddonVersion, CopyTable;
	do -- Just lets me hide everything in Notepad++ :p
		BreadCrumbs.Dongle = DongleStub("Dongle-1.2"):New("DongleBreadCrumbsTemplate");
		
		-- Setup the Localisation library
		L = LibStub("AceLocale-3.0"):GetLocale(BreadCrumbs.Name, true);
		-- Just to store this one until we use it
		local GetAddonVersion;
		-- Get some useful functions and store them locally to make it easyer to use them
		TableCount, GetAddonVersion, CopyTable = LibStub("LibZasUsefulFuncs-0.1"):GetFunctions("TableCount", "GetAddonVersion", "CopyTable");
		-- Pull back the addon version and version type
		BreadCrumbs.CurAddOnVer, BreadCrumbs.CurAddOnVerType, BreadCrumbs.AddonVerTypeColour = GetAddonVersion(BreadCrumbs.Name);
		-- This is used to create and control the options windows in the Bliz Interface window
		AceConfigDialog = LibStub("AceConfigDialog-3.0");
		AceConfigRegistry = LibStub("AceConfigRegistry-3.0");
		-- This library helps you create GUI like the ones used in the Bliz Interface window
		AceGUI = LibStub("AceGUI-3.0");
		-- This library controls the placement and movement of icons on the Minimap
		Astrolabe = DongleStub("Astrolabe-1.0");
		-- This library allows for easier hooking of other addon's functions
		AceHook = LibStub("AceHook-3.0");
		
		-------------------------------------------------------------------------------------------------------------------
		-- Library: Zasurus' Message Control
		-------------------------------------------------------------------------------------------------------------------
			Msg, ErrMsg, DBugMsg, ErrDBugMsg, InitDBug, ZMC_COLOURS, ZMC_Settings = LibStub("LibZasMsgCtr-1.1"):Initialize(BreadCrumbs, nil, "COLOUR_ORANGE");
			
			-- Store the coloured version of "Zasurus"
			ZasName = ZMC_COLOURS.BLUE.."Zas"..ZMC_COLOURS.CLOSE..ZMC_COLOURS.LTBLUE.."ur"..ZMC_COLOURS.CLOSE..ZMC_COLOURS.LTGREEN.."us"..ZMC_COLOURS.CLOSE;
            ZasShortName = ZMC_COLOURS.BLUE.."Z"..ZMC_COLOURS.CLOSE..ZMC_COLOURS.LTBLUE.."a"..ZMC_COLOURS.CLOSE..ZMC_COLOURS.LTGREEN.."s"..ZMC_COLOURS.CLOSE;
			-- Store the coloured version of the addon's name
			BreadCrumbs.ColName = ZMC_Settings.DefaultAddonNameColour..BreadCrumbs.Name..ZMC_COLOURS.CLOSE;
            BreadCrumbs.OptionName = strsub(BreadCrumbs.Name,1,1)..ZMC_Settings.DefaultAddonNameColour..strsub(BreadCrumbs.Name,2)..ZMC_COLOURS.CLOSE.."("..ZasName..")";
			-- Store the colour of the addon version
			AddonVersion = ZMC_COLOURS[BreadCrumbs.AddonVerTypeColour]..strupper(BreadCrumbs.Version).." "..BreadCrumbs.CurAddOnVerType..ZMC_COLOURS.CLOSE;
			
			-----------------------------------------------------------
			-- Valid Msg Formats
			--[[-------------------------------------------------------
				Msg(sMessage[, bSimple, FrameOverride]); -- This is the original format.
				ErrMsg(sMessage[, bSimple, FrameOverride]); -- For displaying standard ERROR messages to the user. (In RED)
				DBugMsg(sMessage[, bSimple, FrameOverride]); -- For displaying debugging messages to the user.
				ErrDBugMsg(sMessage[, bSimple, FrameOverride]); -- For displaying debugging ERROR messages to the user. (In RED)
			-----------------------------------------------------------
			
			-----------------------------------------------------------
			-- AddOn Setting Values
			-----------------------------------------------------------
				ZMC_Settings.DefaultAddonNameColour = Default Colour for the AddOn's Name/Alias Before Standard Messages
				ZMC_Settings.DefaultMsgColour = Default Colour for Standard Messages
				ZMC_Settings.DefaultDBugMsgColour = Default Colour for Standard Debug Messages
				ZMC_Settings.DefaultErrorMsgColour = Default Colour for Standard Err Messages
				ZMC_Settings.DefaultDBugErrorMsgColour = Default Colour for Debug Err Messages
				
				ZMC_Settings.MsgFrameNum = Sets the frame that messages will be DBugMsged in by default (If not overwritten on calling msg function).
				ZMC_Settings.DebugMsgFrameNum = Stores the addon Name for the msg function. This is what is DBugMsged at the start of the message
			--]]-------------------------------------------------------
		-------------------------------------------------------------------------------------------------------------------
	end;
-----------------------------------------------------------------

-----------------------------------------------------------------
-- KeyBinding Variables
-----------------------------------------------------------------
	do
		BINDING_HEADER_BreadCrumbs = "BreadCrumbs";
		BINDING_NAME_BreadCrumbs_ToggleHideBread = L["Toggle Visibility"];
		BINDING_NAME_BreadCrumbs_ToggleHideBreadCurrent = L["Toggle Visibility Override for current 3rd Party Addon/Minimap"];
		BINDING_NAME_BreadCrumbs_ToggleBreadDrop = L["Start/Stop Dropping BreadCrumbs"];
		BINDING_NAME_BreadCrumbs_ToggleDropWhenDead = L["Start/Stop Dropping BreadCrumbs when Dead"];
		BINDING_NAME_BreadCrumbs_OpenOptionsMenu = L["Opens BreadCrumbs's Options Menu"];
	end;
-----------------------------------------------------------------

-- Runs when any event's we are registered for occure
function BreadCrumbs:OnEvent(Event, Arg1, Arg2)
	-------------------------------------------------
	-- ZMC Debug: If bDBug is set to true all of the
	-- debug msgs in THIS function will apear!
	-------------------------------------------------
	-- local bDBug = true; -- Uncomment this to debug
	local DBugMsg, ErrDBugMsg, DBState = InitDBug(bDBug);
	-------------------------------------------------
	
	DBugMsg("BreadCrumbs:OnEvent("..tostring(Event)..") self.LoadedStatus[Initialized] = "..tostring(self.LoadedStatus["Initialized"]).." Arg1(AddOnName) = "..tostring(Arg1).. " and self.LoadedStatus[RunLevel] = "..tostring(self.LoadedStatus["RunLevel"]));
	-- This addon has all been loaded into memmory
	if (Event == "ADDON_LOADED" and self.LoadedStatus["Initialized"] == 0 and Arg1 == "BreadCrumbs") then
		DBugMsg("Event == ADDON_LOADED and self.LoadedStatus[Initialized] == "..tostring(self.LoadedStatus["Initialized"]).." and Arg1(AddOnName) == BreadCrumbs");
		-- Start initializing the addon
		self:Initialize();
	elseif (Event == "ZONE_CHANGED_NEW_AREA" and self.LoadedStatus["Initialized"] == self.LoadedStatus["RunLevel"]) then
		DBugMsg("Changed Zone: Event = ZONE_CHANGED_NEW_AREA");
		-- Removes all minimap icons and readds them
		self:RefreshPoints();
		-- Removes all minimap icons and readds them after a specified time (in seconds)
		self.Dongle:ScheduleTimer("MiniHugeHUD_GoneInside", function() self:RefreshPoints(); end, 1);
	end;
end;

-- Initialize the addon
function BreadCrumbs:Initialize()
	-------------------------------------------------
	-- ZMC Debug: If bDBug is set to true all of the
	-- debug msgs in THIS function will apear!
	-------------------------------------------------
	-- local bDBug = true; -- Uncomment this to debug
	local DBugMsg, ErrDBugMsg, DBState = InitDBug(bDBug);
	-------------------------------------------------
	
	DBugMsg("BreadCrumbs:Initialize()");
	
	-- Stops monitoring the "ADDON_LOADED" event as we don't need it any more
	self:UnregisterEvent("ADDON_LOADED");
	
	if (((BreadCrumbs.CurAddOnVerType == "ALPHA") or (BreadCrumbs.CurAddOnVerType == "DEV")) and not(BreadCrumbs_Options and BreadCrumbs_Options["ALPHA_OK"])) then
		---------------------------------------------------------
		-- ALPHA Warning Static Popup Dialog
		---------------------------------------------------------
			StaticPopupDialogs["BreadCrumbssALPHAStartup"] = {
				text = L["WARNING: This is an"].." "..ZMC_COLOURS.RED..tostring(BreadCrumbs.CurAddOnVerType)..ZMC_COLOURS.CLOSE.." "..L["version of"].." "..ZMC_Settings.DefaultAddonNameColour.."BreadCrumbs!\n"..ZMC_COLOURS.CLOSE..ZMC_COLOURS.ORANGE..L["You should only be running this if you know what that means! Your stored data could be lost!"]..ZMC_COLOURS.CLOSE.."\n"..L["If you are unsure then"]..":\n"..L["Please go to Curse.com and download the newest Release version of"]..ZMC_Settings.DefaultAddonNameColour.." BreadCrumbs"..ZMC_COLOURS.CLOSE..ZMC_COLOURS.ORANGE.." ("..L["or if you want to live on the edge the BETA version)."]..ZMC_COLOURS.CLOSE.."\n\n"..ZMC_COLOURS.GREEN..L["Do you want to continue?"]..ZMC_COLOURS.CLOSE,
				button1 = L["Yes, Always!"],
				button2 = L["Yes, This time!"],
				button3 = L["NO SAVE ME!"],
				OnAccept = function()
					-- Initialize BreadCrumbs_Options encase it doesn't exist
					BreadCrumbs_Options = BreadCrumbs_Options or {};
					-- Disable ALPHA messages
					BreadCrumbs_Options["ALPHA_OK"] = true;
					-- Start Second section of setup.
					self:InitLevel(1);
				end,
				OnCancel = function()
					-- Start Second section of setup.
					self:InitLevel(1);
				end,
				OnAlt = function ()
					ErrMsg(L["Please go to Curse.com and download the newest Release version of"].."BreadCrumbs("..L["or if you want to live on the edge the BETA version)."]);
				end,
				timeout = 0,
				whileDead = true,
				hideOnEscape = true,
			}
		---------------------------------------------------------
		
		-- Shows the Reset BreadCrumbs Settings warning question dialog box
		StaticPopup_Show ("BreadCrumbssALPHAStartup");
		-- Exit and if they want to continue the Static Popup Dialog will run the code.
		return;
	else
		self:InitLevel(1); -- Start Second section of setup.
	end;
end;

-- Initializes the addon to the level specified (various different addon setup levels)
function BreadCrumbs:InitLevel(Level)
	-------------------------------------------------
	-- ZMC Debug: If bDBug is set to true all of the
	-- debug msgs in THIS function will apear!
	-------------------------------------------------
	-- local bDBug = true; -- Uncomment this to debug
	local DBugMsg, ErrDBugMsg, DBState = InitDBug(bDBug);
	-------------------------------------------------
	
	DBugMsg("BreadCrumbs:InitLevel(Level = "..tostring(Level)..")");
	
	DBugMsg("Request to initializing addon from level  = '"..tostring(self.LoadedStatus["Initialized"]).."' to level = '"..tostring(Level).."'");
	
	if (Level == 1) and (self.LoadedStatus["Initialized"] == 0) then
		DBugMsg("Setting Level to 1...");
		-- Initialize the variables
		self:VarInit();
		
		-- Updates addon initialiation to specified level
		self.LoadedStatus["Initialized"] = Level;
		
		--- Slash Command Handler
		SLASH_BreadCrumbs1 = "/BC";
		SLASH_BreadCrumbs = "/BreadCrumbs";
		SlashCmdList["BreadCrumbs"] = function(msg)
			self:SlashCmdHandler(msg);
		end;
		
		-- Creates and adds the options window to the Bliz interface tab
		self:AddInterfaceOptions();
		
		-- Addon disabled so don't do anything else
		if not(BreadCrumbs_Options.Enabled) then
			-- Warn the user if they have the addon disabled (to try and stop user reporting errors!)
			ErrMsg(L["WARNING: Addon disabled! Use"]..": "..tostring(SLASH_BreadCrumbs1).." "..L["enable"]);
		else
			-- Start Second section of setup.
			self:InitLevel(2);
		end;
	elseif (Level == 2) and (self.LoadedStatus["Initialized"] == 1) then
		DBugMsg("Setting Level to 2...");
		-- Updates addon initialiation to specified level
		self.LoadedStatus["Initialized"] = Level;
		-- Ether the current db doesn't match the current version or there hasn't been an update to the db since the last update
		if (not(BreadCrumbs_Options.CurrentDBVersion == self.Version)) then
			DBugMsg("DB Updated");
			
			if ((BreadCrumbs_Options.CurrentDBVersion < self.Version)) then
				-- Warn the user that they have updated to a newer version of BreadCrumbs so if they have problems they should reset all the BreadCrumbs settings
				ErrMsg(L["WARNING: You have updated"].." BreadCrumbs "..L["to a new version. If you have any trouble try resetting the settings to default using the command"].." '"..tostring(SLASH_BreadCrumbs1).." "..L["reset"].."'.");
			elseif (BreadCrumbs_Options.CurrentDBVersion > self.Version) then
				-- Warn the user that they have DOWNGRADED to a older version of BreadCrumbs than there DB so if they have problems they should reset all the BreadCrumbs settings
				Msg(L["WARNING: You have DOWNGRADED BreadCrumbs to a version OLDER than your Database! This is NOT advised as the DB structure MAY have changed... If you have any trouble try resetting the settings to default using the command"].." '"..tostring(SLASH_BreadCrumbs1).." "..L["resetALL'. To dismiss this message use the command"].." '"..tostring(SLASH_BreadCrumbs1).." "..L["DownGradedOK"].."'.", false, DebugTxt, true);
			end;
		else
			DBugMsg("BreadCrumbs_Options.CurrentDBVersion = "..tostring(BreadCrumbs_Options.CurrentDBVersion));
		end;
		-- Change the DB version to be this version...
		BreadCrumbs_Options.CurrentDBVersion = self.Version;
		
		-- Setup the update script so we have a function that runs thousands of times a second for calculations
		self:SetScript("OnUpdate", function(frame, elapsed) BreadCrumbs:Update(frame, elapsed); end)
		
		-- Initializes self.Points if it doesn't already exist
		if ((TableCount(BreadCrumbs_Char_Options.MovementHistory) == 0) or BreadCrumbs_Options.ResetOnReload) then
			DBugMsg("No points so generate them");
			-- Creates the frames (points) that will be used as the points on the map
			self:GeneratePoints();
		else -- There are already points so just restart 
			DBugMsg("There are already points so just restart");
			-- Removes and then readds the points to the minimap etc...
			self:ReAddPoints();
		end;
		
		-- Set the point's visability
		local CurrentHiddenState;
		if (BreadCrumbs_Options.IndividualSettings[self.ActiveMinimapsName].Hidden == nil) then
            DBugMsg("BreadCrumbs_Options.IndividualSettings[self.ActiveMinimapsName].Hidden == nil");
			CurrentHiddenState = BreadCrumbs_Options.Hidden;
		else
            DBugMsg("BreadCrumbs_Options.IndividualSettings[self.ActiveMinimapsName].Hidden (Not nil) = '"..tostring(BreadCrumbs_Options.IndividualSettings[self.ActiveMinimapsName].Hidden).."'");
			CurrentHiddenState = BreadCrumbs_Options.IndividualSettings[self.ActiveMinimapsName].Hidden;
		end;
		self:TogHideCrumbs(CurrentHiddenState);
		
		-- If the SexyMap HUD exists then hook it's show and hide scripts
		if (HudMapCluster) then
			AceHook:HookScript(HudMapCluster, "OnShow", function()BreadCrumbs:HudMapCluster_OnShow(); end)
			AceHook:HookScript(HudMapCluster, "OnHide", function() BreadCrumbs:HudMapCluster_OnHide(); end)
		end;
		
		-- Check if any addon's tried to move us to another minimap frame before we finished loading
		if (self.ReparentToMinimap and not(self.ReparentToMinimap == minimapParent)) then
			-- Resend the request to reparent the minimap
			self:ReparentMinimap(self.ReparentToMinimap, self.ReparentToCallingAddonName, self.ReparentToCallingAddonDetails);
			-- Wipe the variable as it's not needed.
			self.ReparentToMinimap = nil;
		end;
		
		-- Does MinimapButton Frame exist
		if (MBF) then
			-- Ensures that our point name is added to Minimap Button Frames exception list
			self:AddToMBFException();
            -- Ensures that our point name is removed from Minimap Button Frames button override list
            self:RemoveFromMBFOverride();
		end;
		
		-- Setup Check for Zone changes as some times we need to reinitalize 3rd party addons
		self:RegisterEvent("ZONE_CHANGED_NEW_AREA");
		
		-- Display the loaded message to the user
		Msg(L["Loaded"]..". "..L["For help type"].." "..tostring(SLASH_BreadCrumbs1).." "..L["or"].." "..tostring(SLASH_BreadCrumbs));
	else
		ErrMsg(L["Addon's current level"].." ("..tostring(self.LoadedStatus["Initialized"])..") "..L["doesn't match expected level to progress to level"].." ("..tostring(Level)..") "..L["or requested level isn't known... Aborting"].."!'");
	end;
end;

-- Initialize the variables (CONTAINS DEFAULTS AS WELL)
function BreadCrumbs:VarInit()
	-------------------------------------------------
	-- ZMC Debug: If bDBug is set to true all of the
	-- debug msgs in THIS function will apear!
	-------------------------------------------------
	-- local bDBug = true; -- Uncomment this to debug
	local DBugMsg, ErrDBugMsg, DBState = InitDBug(bDBug);
	-------------------------------------------------
	
	DBugMsg("BreadCrumbs:VarInit()");
	-- Initializes BreadCrumbs_Options if it doesn't already exist
	BreadCrumbs_Options = BreadCrumbs_Options or {};
	-- Initializes BreadCrumbs_Char_Options if it doesn't already exist
	BreadCrumbs_Char_Options = BreadCrumbs_Char_Options or {};
	-- Initializes BreadCrumbs_Options.NumPoints with the default value if it doesn't already exist
	BreadCrumbs_Options.NumPoints = BreadCrumbs_Options.NumPoints or 100;
	-- Ensures there are at lest 2 points to stop divide by zero errors
	if (BreadCrumbs_Options.NumPoints < 2) then
		BreadCrumbs_Options.NumPoints = 2;
	end;
	
	-- Initialzes the name of the points
	self.PointsName = "BreadCrumbsPoints"
	
	--------------------------------------------------------------------------
	-- Colour, Opacity & Width
	--------------------------------------------------------------------------
		-- Initialize the Minimap & 3rd Party Addon Settings storage table
		BreadCrumbs_Options.IndividualSettings = BreadCrumbs_Options.IndividualSettings or {};
		-- Initialize the Default Minimap
		BreadCrumbs_Options.IndividualSettings.Minimap = BreadCrumbs_Options.IndividualSettings.Minimap or {};
		-- Initialize Any Unknown Addons settings
		BreadCrumbs_Options.IndividualSettings.AnyUnknownAddons = BreadCrumbs_Options.IndividualSettings.AnyUnknownAddons or {};
		
		-- Loop though all of the Minimaps and 3rd party addon's we know about
		for name, value in pairs(BreadCrumbs_Options.IndividualSettings) do
			-- Initializes Minimap & Addon's Individual Settings Defaults
			self:InitIndividualSetting(name);
		end;
	--------------------------------------------------------------------------
	
	--------------------------------------------------------
	-- Initializes arrays for Storing Gradiant Changes
	--------------------------------------------------------
		self.RedColourGradArray = {};
		self.GreenColourGradArray = {};
		self.BlueColourGradArray = {};
		self.TextureOpacityGradArray = {};
		self.DiameterGradArray = {};
	--------------------------------------------------------
	
	-- Initializes initial setting for active minimap (Always minimap as it's moved after load)
	self.ActiveMinimapsName = "Minimap";
	
	BreadCrumbs_Options.DistBetweenPoints = BreadCrumbs_Options.DistBetweenPoints or 10;
	BreadCrumbs_Char_Options.MovementHistory = BreadCrumbs_Char_Options.MovementHistory or {};
	
	-- Initializes BreadCrumbs_Options.Hidden with the default value if it doesn't already exist
	if (BreadCrumbs_Options.Hidden == nil) then
		BreadCrumbs_Options.Hidden = false;
	end;
	-- Initializes the current points hidden state
	if (BreadCrumbs_Options.IndividualSettings[self.ActiveMinimapsName].Hidden == nil) then
		self.CurrentlyHidden = BreadCrumbs_Options.Hidden;
	else
		self.CurrentlyHidden = BreadCrumbs_Options.IndividualSettings[self.ActiveMinimapsName].Hidden;
	end;
	-- Initializes BreadCrumbs_Options.Dropping with the default value if it doesn't already exist
	if (BreadCrumbs_Options.Dropping == nil) then
		BreadCrumbs_Options.Dropping = true;
	end;
	
	-- Initializes BreadCrumbs_Options.Enabled with the default value if it doesn't already exist
	if (BreadCrumbs_Options.Enabled == nil) then
		BreadCrumbs_Options.Enabled = true;
	end;
	
	-- Initializes BreadCrumbs_Options.DontDropWhenDead with the default value if it doesn't already exist
	if (BreadCrumbs_Options.DontDropWhenDead == nil) then
		BreadCrumbs_Options.DontDropWhenDead = true;
	end;
	
	-- Initializes BreadCrumbs_Options.ResetOnReload with the default value if it doesn't already exist
	if (BreadCrumbs_Options.ResetOnReload == nil) then
		BreadCrumbs_Options.ResetOnReload = false;
	end;
	
	-- Initializes BreadCrumbs_Options.CurrentDBVersion with the default value of self.Version if it doesn't already exist
	if (BreadCrumbs_Options.CurrentDBVersion == nil) then
		BreadCrumbs_Options.CurrentDBVersion = self.Version;
	end;
	
	BreadCrumbs_Options.Texture = [[Interface\AddOns\BreadCrumbs\Artwork\PointBlob.tga]];
	
	-- Resets the time since the last update was performed
	self.last_update = 0;
	
	-- Say that the addon variables have now all been initialized
	self.LoadedStatus["AddonVariables"] = true;
end

-- Slash Command Handler
function BreadCrumbs:SlashCmdHandler(msg)
	-------------------------------------------------
	-- ZMC Debug: If bDBug is set to true all of the
	-- debug msgs in THIS function will apear!
	-------------------------------------------------
	-- local bDBug = true; -- Uncomment this to debug
	local DBugMsg, ErrDBugMsg, DBState = InitDBug(bDBug);
	-------------------------------------------------
	
	DBugMsg("BreadCrumbs:SlashCmdHandler(msg = "..tostring(msg)..")");
	
	--------------------------------------------------------------
	-- Stores the Option List in a string
	--------------------------------------------------------------
		local OptionList = "";
		OptionList = OptionList.."- "..L["options"].." - "..L["Opens up the options Menu for"].." BreadCrumbs";
		OptionList = OptionList.."\n- "..L["reset"].." - "..L["Resets ALL settings! Like you just installed the addon. Good for last attempt at fixing a problem with the addon!"]
		OptionList = OptionList.."\n- "..L["DownGradedOK"].." - "..L["Hides the downgrade warning at the start-up (if you have downgraded to an earlier version of this addon and everything is OK AND you WANT to keep this old version!)"]
		OptionList = OptionList.."\n- "..L["enable"].." - "..L["Enables this addon"]
		OptionList = OptionList.."\n- "..L["disable"].." - "..L["Disables this addon"].." ("..L["If you are disabling this addon as you don't intend to use it for a long period you are better disabling it from the character selection screen to save memory!"]..")"
	--------------------------------------------------------------
	
	if (msg == nil or msg == "") then
		-- Tell the user the addon's enabled state
		if (BreadCrumbs_Options.Enabled) then
			Msg(L["is Enabled"]);
		else
			ErrMsg(L["is Disabled"]);
		end;
		-- Tell the user the trail's enabled state
		if self.CurrentlyHidden then
			ErrMsg(L["Trail is Hidden"]);
		else
			Msg(L["Trail is Visible"]);
		end;
		-- Tell the user BreadCrumb's dropping state
		if (BreadCrumbs_Options.Dropping) then
			Msg(L["BreadCrumb Dropping Enabled"]);
		else
			ErrMsg(L["BreadCrumb Dropping Disabled"]);
		end;
		-- Tell the user ""Don't Drop When Dead"'s enabled state
		if (BreadCrumbs_Options.DontDropWhenDead) then
			Msg(L["Don't Drop When Dead Enabled"]);
		else
			ErrMsg(L["Don't Drop When Dead Disabled"]);
		end;
		
		Msg(L["Use"].." '"..tostring(SLASH_BreadCrumbs1).." "..L["options"].."' "..L["to open up the Addon's Options screen"].."\n"..OptionList);
	elseif ((strlower(strsub(msg,1,7)) == L["options"]) or (strlower(strsub(msg,1,6)) == "config")) then -- This is the macro checking in...
		Msg(L["Opening BreadCrumbs Options Panel"]);
		
		InterfaceOptionsFrame_OpenToCategory(self.OptionsFrame);
	elseif(strlower(strsub(msg,1,5)) == L["reset"]) then -- This is the macro checking in...
		Msg(L["Reloading Defaults"]);
		-- Shows the Reset BreadCrumbs Settings warning question dialog box
		StaticPopup_Show ("ResetBreadCrumbssSettings");
	elseif(strlower(strsub(msg,1,12)) == strlower(L["DownGradedOK"])) then
		Msg(L["Downgrade warning dismissed"]);
		
		BreadCrumbs_Options.CurrentDBVersion = self.Version
	elseif(strlower(strsub(msg,1,6)) == L["enable"]) then -- This is the macro checking in...
		Msg(L["Enabling BreadCrumbs"]);
		
		self:TogAddon(true);
	elseif(strlower(strsub(msg,1,7)) == L["disable"]) then -- This is the macro checking in...
		Msg(L["Disabling BreadCrumbs"]);
		
		self:TogAddon(false);
	elseif(strlower(strsub(msg,1,7)) == "alphaok") then -- This is the macro checking in...
		-- Invert the current status of BreadCrumbs_Options["ALPHA_OK"]
		BreadCrumbs_Options["ALPHA_OK"] = not(BreadCrumbs_Options["ALPHA_OK"]);
		Msg(L["ALPHA warning changed to"]..": "..tostring(BreadCrumbs_Options["ALPHA_OK"]).."!");
	else
		ErrMsg(L["Message not known"].." = '"..tostring(msg).."' "..L["Valid options are"]..":\n"..OptionList);
	end;
end;

-- Creates and adds the options window to the Bliz interface tab
function BreadCrumbs:AddInterfaceOptions()
	-------------------------------------------------
	-- ZMC Debug: If bDBug is set to true all of the
	-- debug msgs in THIS function will apear!
	-------------------------------------------------
	-- local bDBug = true; -- Uncomment this to debug
	local DBugMsg, ErrDBugMsg, DBState = InitDBug(bDBug);
	-------------------------------------------------
	
	DBugMsg("BreadCrumbs:AddInterfaceOptions()");
	
	-- Initializes the reset button to disabled
	local ResetButtonEnabled = false;
	
	---------------------------------------------------------
	-- Reset Settings Static Popup Dialog
	---------------------------------------------------------
		StaticPopupDialogs["ResetBreadCrumbssSettings"] = {
			text = L["Are you sure you want to reset ALL BreadCrumbs Settings to default?"],
			button1 = "Yes",
			button2 = "No",
			OnAccept = function()
				self:ResetAllSettings(); -- Deletes all settings and restarts UI
			end,
			OnCancel = function()
				ResetButtonEnabled = false; -- Disables the button again
				AceConfigRegistry:NotifyChange(self.Name) -- Refreshes Options Window
			end,
			timeout = 0,
			whileDead = true,
			hideOnEscape = true,
		}
	---------------------------------------------------------
	
	---------------------------------------------------------
	-- First Time Setup Static Popup Dialog
	---------------------------------------------------------
		StaticPopupDialogs["SetupBreadCrumbs"] = {
			text = L["As this appears to be the first time you have used"].." BreadCrumbs "..L["Would you like to open the options menu and setup the Addon like you want it?"].." "..ZMC_COLOURS.ORANGE..L["There are loads of good settings"].."! ;)"..ZMC_COLOURS.CLOSE,
			button1 = L["Yes"],
			button2 = L["No"],
			OnAccept = function()
				-- Open the BreadCrumbs's options menu
				InterfaceOptionsFrame_OpenToCategory(self.OptionsFrame);
			end,
			timeout = 0,
			whileDead = true,
			hideOnEscape = true,
		}
	---------------------------------------------------------
	
	---------------------------------------------------------
	-- Number of Crumbs Manual Override Static Popup Dialog
	---------------------------------------------------------
		StaticPopupDialogs["NumCrumbsManualOverride"] = {
			text = L["Enter number of crumbs to use"],
			button1 = L["Yes"],
			button2 = L["No"],
			timeout = 0,
			hasEditBox = true,
			maxLetters = 32,
			whileDead = true,
			hideOnEscape = true,
			OnAccept = function(self)
				-- Updates/sets the number of points to use and updates everything
				self:UpdateNumberOfPoints(self.editBox:GetText());
			end,
			EditBoxOnEnterPressed = function(self, data)
				local parent = self:GetParent();
				-- Updates/sets the number of points to use and updates everything
				BreadCrumbs:UpdateNumberOfPoints(parent.editBox:GetText());
				parent:Hide();
				-- Update the Config window to show changes
				AceConfigRegistry:NotifyChange(BreadCrumbs.Name);
			end,
			EditBoxOnEscapePressed = function (self)
				self:GetParent():Hide();
			end,
			OnHide = function (self)
				self.editBox:SetText("");
			end,
		}
	---------------------------------------------------------
	
	-- This is the options config array for BreadCrumbs's Options screen	
	self.Options = {
		type = "group",
		childGroups = "tab",
		name = ZMC_Settings.DefaultAddonNameColour..tostring(self.Name)..ZMC_COLOURS.CLOSE.."("..ZasName..") - "..AddonVersion,
		desc = L["These options allow you to configure various aspects of BreadCrumbs."],
		args = {
			-----------------------------------------------------------------------------------------------------------
			-- General
			---------------------------------------------------------------------------------------------------------------
				GeneralGroup = {
					type = "group",
					name = L["General Settings"],
					order = 1,
					args = {
						-- Creates and Sets Up the Addon Enable Toggle
						Enabled = {
							type = "toggle",
							name = L["BreadCrumbs Enabled (Key-binding Available)"],
							desc = L["Turns BreadCrumbs On/Off. If you don't plan to use it for a long time or just don't want to use it on this toon your better disabling it via the 'Addon' menu on the toon selection screen to save memory."],
							order = 1,
							get = function(info)
								return BreadCrumbs_Options.Enabled;
							end,
							set = function(info, value)
								self:TogAddon(value);
							end,
							width = "full",
						},
						-- Creates and Sets Up the Addon Drop Toggle
						Drop = {
							type = "toggle",
							name = L["Drop Breadcrumbs (Key-binding Available)"],
							desc = L["Drop BreadCrumbs On/Off. This starts and stops the dropping of breadcrumbs."],
							order = 2,
							get = function(info)
								return BreadCrumbs_Options.Dropping;
							end,
							set = function(info, value)
								self:TogBreadDrop(value);
							end,
							width = "full",
						},
						-- Creates and Sets Up the Addon Hide Toggle
						HideTog = {
							type = "toggle",
							name = L["Hide Breadcrumbs (Key-binding Available)"],
							desc = L["Hides or un-hides the breadcrumbs. This doesn't stop the dropping just hides them. This can be overwritten from the 3rd party addon/minimap specific settings"],
							order = 3,
							get = function(info)
								return BreadCrumbs_Options.Hidden;
							end,
							set = function(info, value)
								BreadCrumbs_Options.Hidden = value;
								
								local CurrentHiddenState;
								if (BreadCrumbs_Options.IndividualSettings[self.ActiveMinimapsName].Hidden == nil) then
									CurrentHiddenState = BreadCrumbs_Options.Hidden;
								else
									CurrentHiddenState = BreadCrumbs_Options.IndividualSettings[self.ActiveMinimapsName].Hidden;
								end;
								
								self:TogHideCrumbs(CurrentHiddenState);
							end,
							width = "full",
						},
						-- Creates and Sets Up the Addon "Reset points on reload/load" Toggle
						ResetOnReload = {
							type = "toggle",
							name = L["Reset Trail on Load/Reload"],
							desc = L["Enable to start a fresh trail every time the addon is loaded/reloaded."],
							order = 4,
							get = function(info)
								return BreadCrumbs_Options.ResetOnReload;
							end,
							set = function(info, value)
								BreadCrumbs_Options.ResetOnReload = value;
							end,
							width = "full",
						},
						-- Creates and Sets Up the Addon "Don't Drop when Dead" Toggle
						DontDropWhenDead = {
							type = "toggle",
							name = L["Don't Drop Breadcrumbs when Dead (Key-binding Available)"],
							desc = L["Enable to stops dropping BreadCrumbs when you die so you don't have a useless trail from the grave and lose your bearings!"],
							order = 5,
							get = function(info)
								return BreadCrumbs_Options.DontDropWhenDead;
							end,
							set = function(info, value)
								self:TogDropWhenDead(value);
							end,
							width = "full",
						},
						-- Creates and Sets Up the DistBetweenPoints slider
						DistBetweenPoints = {
							type = "range",
							name = L["Space Between Crumbs"],
							desc = L["This slider"].." "..L["specifies how close together the crumbs are that make up the line. Closer together gives a better looking line but makes it shorter."].."\n\n"..ZMC_Settings.DefaultAddonNameColour..L["NOTE: This will also affect the length of the line so you may need to also adjust the number of crumbs used for the line."].."\n\n"..L["Also this will only affect new BreadCrumbs and not your existing trail."].."|r",
							order = 6,
							min = 1,
							max = 100,
							step = 1,
							get = function(info)
								return BreadCrumbs_Options.DistBetweenPoints;
							end,
							set = function(info, value)
								BreadCrumbs_Options.DistBetweenPoints = value;
							end,
							
						},
						NewLine_1 = {
							order	= 6.5,
							name	= "",
							type	= "description",
						},
						-- Creates and Sets Up the NumberOfPoints slider
						NumberOfPoints = {
							type = "range",
							name = L["Number of Crumbs"],
							desc = L["This slider"].." "..L["specifies how many crumbs are used to create the line and therefore how long it will be."].."\n\n"..ZMC_Settings.DefaultAddonNameColour..L["NOTE: Use this and the spacing one to crate the line you want."].."\n\n"..L["ALSO this will reset your BreadCrumbs(remove your trail)! So ensure you make a note of which way you have come from."].."|r\n\n"..ZMC_Settings.DefaultErrorMsgColour..L["WARNING! Setting this higher MAY slow the game down!"].."|r",
							order = 7,
							-- Can't be <2!
							min = 2,
							max = 200,
							step = 1,
							get = function(info)
								return BreadCrumbs_Options.NumPoints;
							end,
							set = function(info, value)
								-- Updates/sets the number of points to use and updates everything
								self:UpdateNumberOfPoints(value);
							end,
						},
						-- Manually enter number of points
						NumberOfPointsManual = {
							type = "execute",
							name = L["Manual Override"],
							desc = L["Allows you to manually"].." "..L["specifies how many crumbs are used to create the line and therefore how long it will be."].."\n\n"..ZMC_Settings.DefaultAddonNameColour..L["NOTE: Use this and the spacing one to crate the line you want."].."\n\n"..L["ALSO this will reset your BreadCrumbs(remove your trail)! So ensure you make a note of which way you have come from."].."|r\n\n"..ZMC_Settings.DefaultErrorMsgColour..L["WARNING! Setting this higher MAY slow the game down!"].."|r",
							order = 7.4,
							func = function ()
								-- Show the popup to request the manual number and warn the user.
								StaticPopup_Show ("NumCrumbsManualOverride");
							end,
						},
						NewLine_2 = {
							order	= 7.5,
							name	= "",
							type	= "description",
						},
						-- Creates and Sets Up the HUD Size Gradiant Toggle
						EnableReset = {
							type = "toggle",
							name = L["Enable Reset"],
							desc = L["This enables the reset button (so you don't click it by accident!)"],
							order = 9,
							get = function(info)
								return ResetButtonEnabled;
							end,
							set = function(info, value)
								ResetButtonEnabled = value;
							end,
						},
						-- Creates and Sets up the Reset Settings Button
						ResetButton = {
							type = "execute",
							name = L["Reset Settings"],
							desc = L["This resets ALL BreadCrumbs settings back to there defaults (Like a fresh install)!"].."\n\n"..ZMC_COLOURS.ORANGE..L["WARNING! This will reload the UI!"]..ZMC_COLOURS.CLOSE,
							order = 10,
							func = function(info)
								-- Shows the Reset BreadCrumbs Settings warning question dialog box
								StaticPopup_Show ("ResetBreadCrumbssSettings");
							end,
							disabled = function(info)
								return not(ResetButtonEnabled);
							end,
						},
					},
				},
			---------------------------------------------------------------------------------------------------------------
			
			--------------------------------------------------------------------------------------------------------------
			-- KeyBindings
			--------------------------------------------------------------------------------------------------------------
				KeyBindingsGroup = {
					type = "group",
					name = "KeyBindings",
					order = 2,
					args = {
						desc = {
							type = "description",
							order = 0,
							name = L["These are the key-bindings for MiniHugHUD"]
						},
						-----------------------------------------------------------------------------------------------
						-- Keybindings for BreadCrumbs's Visability
						-----------------------------------------------------------------------------------------------
							--------------------------------------------------------
							-- Keybinding 1 for BreadCrumbs
							--------------------------------------------------------
								BindingToggleBCsVisability1 = {
									type = "keybinding",
									name = L["Visibility"].." "..L["Key"].." 1\n",
									desc = L["Toggle BreadCrumb Visibility"].." "..L["Key"].." 1\n",
									order = 1,
									get = function()
										return GetBindingKey("BreadCrumbs_ToggleHideBread");
									end,
									set = function(info, value)
										-- Pull back the first keybinding
										local CurrentKeyBinding1 = GetBindingKey("BreadCrumbs_ToggleHideBread");
										-- Check if the user pressed ESC to wipe the existing keybinding
										if not(value == "") then
											-- Check if there is an existing keybinding
											if (CurrentKeyBinding1) then
												-- Wipe original keybinding
												SetBinding(CurrentKeyBinding1);
											end;
											-- Setup new keybinding
											SetBinding(value, "BreadCrumbs_ToggleHideBread");
											-- Save new keybindings
											SaveBindings(GetCurrentBindingSet());
										-- If ESC was pressed to wipe the existing keybinding ensure there IS existing keybindings
										elseif (CurrentKeyBinding1) then
											-- Wipe original keybinding
											SetBinding(CurrentKeyBinding1)
											-- Save new keybindings
											SaveBindings(GetCurrentBindingSet());
										end;
									end
								},
							--------------------------------------------------------
							
							--------------------------------------------------------
							-- Keybinding 2 for BreadCrumbs
							--------------------------------------------------------
								BindingToggleBCsVisability2 = {
									type = "keybinding",
									name = L["Visibility"].." "..L["Key"].." 2\n",
									desc = L["Toggle BreadCrumb Visibility"].." "..L["Key"].." 2\n",
									order = 2,
									get = function()
										-- Pull back the first two keybindings
										local CurrentKeyBinding1, CurrentKeyBinding2 = GetBindingKey("BreadCrumbs_ToggleHideBread");
										-- Return the second keybinding
										return CurrentKeyBinding2;
									end,
									set = function(info, value)
										-- Pull back the first two keybindings
										local CurrentKeyBinding1, CurrentKeyBinding2 = GetBindingKey("BreadCrumbs_ToggleHideBread");
										-- Check if the user pressed ESC to wipe the existing keybinding
										if not(value == "") then
											-- Check if there is an existing keybinding
											if (CurrentKeyBinding2) then
												-- Wipe original keybinding
												SetBinding(CurrentKeyBinding2);
											end;
											-- Setup new keybinding
											SetBinding(value, "BreadCrumbs_ToggleHideBread");
											-- Save new keybindings
											SaveBindings(GetCurrentBindingSet());
										-- If ESC was pressed to wipe the existing keybinding ensure there IS existing keybindings
										elseif (CurrentKeyBinding2) then
											-- Wipe original keybinding
											SetBinding(CurrentKeyBinding2)
											-- Save new keybindings
											SaveBindings(GetCurrentBindingSet());
										end;
									end
								},
							--------------------------------------------------------
							
							--------------------------------------------------------
							-- Warn user if there are more keybindings than are displayed
							--------------------------------------------------------
								NumToggleBCsVisabilityKeybinding = {
									type = "description",
									order = 3,
									fontSize = "large",
									hidden = function()
										-- Get the number of keybindings
										local NumberOfKeybindings = TableCount({GetBindingKey("BreadCrumbs_ToggleHideBread")});
										-- Get if there are NOT more than 2 as a boolean
										local Hide = not(NumberOfKeybindings > 2);
										-- Return boolean
										return Hide;
									end,
									name = function()
										-- Get the number of keybindings
										local NumberOfKeybindings = TableCount({GetBindingKey("BreadCrumbs_ToggleHideBread")});
										-- 
										if (NumberOfKeybindings > 2) then
											-- Return a warning msg
											return ZMC_COLOURS.RED..L["WARNING! There are"]..": "..tostring(NumberOfKeybindings).." "..L["separate key-bindings for the event"].." '"..L["Visibility"].."'!\n"..L["More than can be displayed here"].."!"..ZMC_COLOURS.CLOSE;
										else
											-- Nothing to worry about so return a blank string
											return "";
										end;
									end
								},
							--------------------------------------------------------
							
							NewLine_1 = {
								order	= 4,
								name	= "",
								type	= "description",
							},
						-----------------------------------------------------------------------------------------------
						
						
						-----------------------------------------------------------------------------------------------
						-- Keybindings for BreadCrumbs's Visability Override
						-----------------------------------------------------------------------------------------------
							--------------------------------------------------------
							-- Keybinding 1 for BreadCrumbs
							--------------------------------------------------------
								BindingToggleBCsVisabilityOverride1 = {
									type = "keybinding",
									name = L["Cur Vis Override"].." "..L["Key"].." 1\n",
									desc = L["Toggle BreadCrumb Current minimap/3rd party addon's Visibility Override"].." "..L["Key"].." 1\n",
									order = 5,
									get = function()
										return GetBindingKey("BreadCrumbs_ToggleHideBreadCurrent");
									end,
									set = function(info, value)
										-- Pull back the first keybinding
										local CurrentKeyBinding1 = GetBindingKey("BreadCrumbs_ToggleHideBreadCurrent");
										-- Check if the user pressed ESC to wipe the existing keybinding
										if not(value == "") then
											-- Check if there is an existing keybinding
											if (CurrentKeyBinding1) then
												-- Wipe original keybinding
												SetBinding(CurrentKeyBinding1);
											end;
											-- Setup new keybinding
											SetBinding(value, "BreadCrumbs_ToggleHideBreadCurrent");
											-- Save new keybindings
											SaveBindings(GetCurrentBindingSet());
										-- If ESC was pressed to wipe the existing keybinding ensure there IS existing keybindings
										elseif (CurrentKeyBinding1) then
											-- Wipe original keybinding
											SetBinding(CurrentKeyBinding1)
											-- Save new keybindings
											SaveBindings(GetCurrentBindingSet());
										end;
									end
								},
							--------------------------------------------------------
							
							--------------------------------------------------------
							-- Keybinding 2 for BreadCrumbs
							--------------------------------------------------------
								BindingToggleBCsVisabilityOverride2 = {
									type = "keybinding",
									name = L["Cur Vis Override"].." "..L["Key"].." 2\n",
									desc = L["Toggle BreadCrumb Current minimap/3rd party addon's Visibility Override"].." "..L["Key"].." 2\n",
									order = 6,
									get = function()
										-- Pull back the first two keybindings
										local CurrentKeyBinding1, CurrentKeyBinding2 = GetBindingKey("BreadCrumbs_ToggleHideBreadCurrent");
										-- Return the second keybinding
										return CurrentKeyBinding2;
									end,
									set = function(info, value)
										-- Pull back the first two keybindings
										local CurrentKeyBinding1, CurrentKeyBinding2 = GetBindingKey("BreadCrumbs_ToggleHideBreadCurrent");
										-- Check if the user pressed ESC to wipe the existing keybinding
										if not(value == "") then
											-- Check if there is an existing keybinding
											if (CurrentKeyBinding2) then
												-- Wipe original keybinding
												SetBinding(CurrentKeyBinding2);
											end;
											-- Setup new keybinding
											SetBinding(value, "BreadCrumbs_ToggleHideBreadCurrent");
											-- Save new keybindings
											SaveBindings(GetCurrentBindingSet());
										-- If ESC was pressed to wipe the existing keybinding ensure there IS existing keybindings
										elseif (CurrentKeyBinding2) then
											-- Wipe original keybinding
											SetBinding(CurrentKeyBinding2)
											-- Save new keybindings
											SaveBindings(GetCurrentBindingSet());
										end;
									end
								},
							--------------------------------------------------------
							
							--------------------------------------------------------
							-- Warn user if there are more keybindings than are displayed
							--------------------------------------------------------
								NumToggleBCsVisabilityOverrideKeybinding = {
									type = "description",
									order = 7,
									fontSize = "large",
									hidden = function()
										-- Get the number of keybindings
										local NumberOfKeybindings = TableCount({GetBindingKey("BreadCrumbs_ToggleHideBreadCurrent")});
										-- Get if there are NOT more than 2 as a boolean
										local Hide = not(NumberOfKeybindings > 2);
										-- Return boolean
										return Hide;
									end,
									name = function()
										-- Get the number of keybindings
										local NumberOfKeybindings = TableCount({GetBindingKey("BreadCrumbs_ToggleHideBreadCurrent")});
										-- 
										if (NumberOfKeybindings > 2) then
											-- Return a warning msg
											return ZMC_COLOURS.RED..L["WARNING! There are"]..": "..tostring(NumberOfKeybindings).." "..L["separate key-bindings for the event"].." '"..L["Visibility"].."'!\n"..L["More than can be displayed here"].."!"..ZMC_COLOURS.CLOSE;
										else
											-- Nothing to worry about so return a blank string
											return "";
										end;
									end
								},
							--------------------------------------------------------
							
							NewLine_2 = {
								order	= 8,
								name	= "",
								type	= "description",
							},
						-----------------------------------------------------------------------------------------------
						
						
						-----------------------------------------------------------------------------------------------
						-- Keybindings for BreadCrumbs's Option Menu
						-----------------------------------------------------------------------------------------------
							--------------------------------------------------------
							-- Keybinding 1 for BreadCrumbs's Toggle BreadCrumb Dropping
							--------------------------------------------------------
								BindingToggleBreadDrop1 = {
									type = "keybinding",
									name = L["BCDropping"].." "..L["Key"].." 1\n",
									desc = L["Toggle BreadCrumb Dropping"].." "..L["Key"].." 1\n",
									order = 9,
									get = function()
										return GetBindingKey("BreadCrumbs_ToggleBreadDrop");
									end,
									set = function(info, value)
										-- Pull back the first keybinding
										local CurrentKeyBinding1 = GetBindingKey("BreadCrumbs_ToggleBreadDrop");
										-- Check if the user pressed ESC to wipe the existing keybinding
										if not(value == "") then
											-- Check if there is an existing keybinding
											if (CurrentKeyBinding1) then
												-- Wipe original keybinding
												SetBinding(CurrentKeyBinding1);
											end;
											-- Setup new keybinding
											SetBinding(value, "BreadCrumbs_ToggleBreadDrop");
											-- Save new keybindings
											SaveBindings(GetCurrentBindingSet());
										-- If ESC was pressed to wipe the existing keybinding ensure there IS existing keybindings
										elseif (CurrentKeyBinding1) then
											-- Wipe original keybinding
											SetBinding(CurrentKeyBinding1)
											-- Save new keybindings
											SaveBindings(GetCurrentBindingSet());
										end;
									end
								},
							--------------------------------------------------------
							
							--------------------------------------------------------
							-- Keybinding 2 for BreadCrumbs's Toggle BreadCrumb Dropping
							--------------------------------------------------------
								BindingToggleBreadDrop2 = {
									type = "keybinding",
									name = L["BCDropping"].." "..L["Key"].." 2\n",
									desc = L["Toggle BreadCrumb Dropping"].." "..L["Key"].." 2\n",
									order = 10,
									get = function()
										-- Pull back the first two keybindings
										local CurrentKeyBinding1, CurrentKeyBinding2 = GetBindingKey("BreadCrumbs_ToggleBreadDrop");
										-- Return the second keybinding
										return CurrentKeyBinding2;
									end,
									set = function(info, value)
										-- Pull back the first two keybindings
										local CurrentKeyBinding1, CurrentKeyBinding2 = GetBindingKey("BreadCrumbs_ToggleBreadDrop");
										-- Check if the user pressed ESC to wipe the existing keybinding
										if not(value == "") then
											-- Check if there is an existing keybinding
											if (CurrentKeyBinding2) then
												-- Wipe original keybinding
												SetBinding(CurrentKeyBinding2);
											end;
											-- Setup new keybinding
											SetBinding(value, "BreadCrumbs_ToggleBreadDrop");
											-- Save new keybindings
											SaveBindings(GetCurrentBindingSet());
										-- If ESC was pressed to wipe the existing keybinding ensure there IS existing keybindings
										elseif (CurrentKeyBinding2) then
											-- Wipe original keybinding
											SetBinding(CurrentKeyBinding2)
											-- Save new keybindings
											SaveBindings(GetCurrentBindingSet());
										end;
									end
								},
							--------------------------------------------------------
							
							--------------------------------------------------------
							-- Warn user if there are more keybindings than are displayed
							--------------------------------------------------------
								NumToggleBreadDropKeybinding = {
									type = "description",
									order = 11,
									fontSize = "large",
									hidden = function()
										-- Get the number of keybindings
										local NumberOfKeybindings = TableCount({GetBindingKey("BreadCrumbs_ToggleBreadDrop")});
										-- Get if there are NOT more than 2 as a boolean
										local Hide = not(NumberOfKeybindings > 2);
										-- Return boolean
										return Hide;
									end,
									name = function()
										-- Get the number of keybindings
										local NumberOfKeybindings = TableCount({GetBindingKey("BreadCrumbs_ToggleBreadDrop")});
										-- 
										if (NumberOfKeybindings > 2) then
											-- Return a warning msg
											return ZMC_COLOURS.RED..L["WARNING! There are"]..": "..tostring(NumberOfKeybindings).." "..L["separate key-bindings for the event"].." '"..L["Options Menu"].."'!\n"..L["More than can be displayed here"].."!"..ZMC_COLOURS.CLOSE;
										else
											-- Nothing to worry about so return a blank string
											return "";
										end;
									end
								},
							--------------------------------------------------------
							
							NewLine_3 = {
								order	= 12,
								name	= "",
								type	= "description",
							},
						-----------------------------------------------------------------------------------------------
						
						
						-----------------------------------------------------------------------------------------------
						-- Keybindings for BreadCrumbs's Option Menu
						-----------------------------------------------------------------------------------------------
							--------------------------------------------------------
							-- Keybinding 1 for BreadCrumbs's Options Menu
							--------------------------------------------------------
								BindingDropWhenDead1 = {
									type = "keybinding",
									name = L["DropWhenDead"].." "..L["Key"].." 1\n",
									desc = L["Toggle Dropping BreadCrumbs When Dead"].." "..L["Key"].." 1\n",
									order = 13,
									get = function()
										return GetBindingKey("BreadCrumbs_ToggleDropWhenDead");
									end,
									set = function(info, value)
										-- Pull back the first keybinding
										local CurrentKeyBinding1 = GetBindingKey("BreadCrumbs_ToggleDropWhenDead");
										-- Check if the user pressed ESC to wipe the existing keybinding
										if not(value == "") then
											-- Check if there is an existing keybinding
											if (CurrentKeyBinding1) then
												-- Wipe original keybinding
												SetBinding(CurrentKeyBinding1);
											end;
											-- Setup new keybinding
											SetBinding(value, "BreadCrumbs_ToggleDropWhenDead");
											-- Save new keybindings
											SaveBindings(GetCurrentBindingSet());
										-- If ESC was pressed to wipe the existing keybinding ensure there IS existing keybindings
										elseif (CurrentKeyBinding1) then
											-- Wipe original keybinding
											SetBinding(CurrentKeyBinding1)
											-- Save new keybindings
											SaveBindings(GetCurrentBindingSet());
										end;
									end
								},
							--------------------------------------------------------
							
							--------------------------------------------------------
							-- Keybinding 2 for BreadCrumbs's Options Menu
							--------------------------------------------------------
								BindingDropWhenDead2 = {
									type = "keybinding",
									name = L["DropWhenDead"].." "..L["Key"].." 2\n",
									desc = L["Toggle Dropping BreadCrumbs When Dead"].." "..L["Key"].." 2\n",
									order = 14,
									get = function()
										-- Pull back the first two keybindings
										local CurrentKeyBinding1, CurrentKeyBinding2 = GetBindingKey("BreadCrumbs_ToggleDropWhenDead");
										-- Return the second keybinding
										return CurrentKeyBinding2;
									end,
									set = function(info, value)
										-- Pull back the first two keybindings
										local CurrentKeyBinding1, CurrentKeyBinding2 = GetBindingKey("BreadCrumbs_ToggleDropWhenDead");
										-- Check if the user pressed ESC to wipe the existing keybinding
										if not(value == "") then
											-- Check if there is an existing keybinding
											if (CurrentKeyBinding2) then
												-- Wipe original keybinding
												SetBinding(CurrentKeyBinding2);
											end;
											-- Setup new keybinding
											SetBinding(value, "BreadCrumbs_ToggleDropWhenDead");
											-- Save new keybindings
											SaveBindings(GetCurrentBindingSet());
										-- If ESC was pressed to wipe the existing keybinding ensure there IS existing keybindings
										elseif (CurrentKeyBinding2) then
											-- Wipe original keybinding
											SetBinding(CurrentKeyBinding2)
											-- Save new keybindings
											SaveBindings(GetCurrentBindingSet());
										end;
									end
								},
							--------------------------------------------------------
							
							--------------------------------------------------------
							-- Warn user if there are more keybindings than are displayed
							--------------------------------------------------------
								NumDropWhenDeadKeybinding = {
									type = "description",
									order = 15,
									fontSize = "large",
									hidden = function()
										-- Get the number of keybindings
										local NumberOfKeybindings = TableCount({GetBindingKey("BreadCrumbs_ToggleDropWhenDead")});
										-- Get if there are NOT more than 2 as a boolean
										local Hide = not(NumberOfKeybindings > 2);
										-- Return boolean
										return Hide;
									end,
									name = function()
										-- Get the number of keybindings
										local NumberOfKeybindings = TableCount({GetBindingKey("BreadCrumbs_ToggleDropWhenDead")});
										-- 
										if (NumberOfKeybindings > 2) then
											-- Return a warning msg
											return ZMC_COLOURS.RED..L["WARNING! There are"]..": "..tostring(NumberOfKeybindings).." "..L["separate key-bindings for the event"].." '"..L["Options Menu"].."'!\n"..L["More than can be displayed here"].."!"..ZMC_COLOURS.CLOSE;
										else
											-- Nothing to worry about so return a blank string
											return "";
										end;
									end
								},
							--------------------------------------------------------
							
							NewLine_4 = {
								order	= 16,
								name	= "",
								type	= "description",
							},
						-----------------------------------------------------------------------------------------------
						
						
						-----------------------------------------------------------------------------------------------
						-- Keybindings for BreadCrumbs's Option Menu
						-----------------------------------------------------------------------------------------------
							--------------------------------------------------------
							-- Keybinding 1 for BreadCrumbs's Options Menu
							--------------------------------------------------------
								BindingOptionMenu1 = {
									type = "keybinding",
									name = L["Options Menu"].." "..L["Key"].." 1\n",
									desc = L["Opens BreadCrumbs Interface Options Window"].." "..L["Key"].." 2\n",
									order = 17,
									get = function()
										return GetBindingKey("BreadCrumbs_OpenOptionsMenu");
									end,
									set = function(info, value)
										-- Pull back the first keybinding
										local CurrentKeyBinding1 = GetBindingKey("BreadCrumbs_OpenOptionsMenu");
										-- Check if the user pressed ESC to wipe the existing keybinding
										if not(value == "") then
											-- Check if there is an existing keybinding
											if (CurrentKeyBinding1) then
												-- Wipe original keybinding
												SetBinding(CurrentKeyBinding1);
											end;
											-- Setup new keybinding
											SetBinding(value, "BreadCrumbs_OpenOptionsMenu");
											-- Save new keybindings
											SaveBindings(GetCurrentBindingSet());
										-- If ESC was pressed to wipe the existing keybinding ensure there IS existing keybindings
										elseif (CurrentKeyBinding1) then
											-- Wipe original keybinding
											SetBinding(CurrentKeyBinding1)
											-- Save new keybindings
											SaveBindings(GetCurrentBindingSet());
										end;
									end
								},
							--------------------------------------------------------
							
							--------------------------------------------------------
							-- Keybinding 2 for BreadCrumbs's Options Menu
							--------------------------------------------------------
								BindingOptionMenu2 = {
									type = "keybinding",
									name = L["Options Menu"].." "..L["Key"].." 2\n",
									desc = L["Opens BreadCrumbs Interface Options Window"].." "..L["Key"].." 2\n",
									order = 18,
									get = function()
										-- Pull back the first two keybindings
										local CurrentKeyBinding1, CurrentKeyBinding2 = GetBindingKey("BreadCrumbs_OpenOptionsMenu");
										-- Return the second keybinding
										return CurrentKeyBinding2;
									end,
									set = function(info, value)
										-- Pull back the first two keybindings
										local CurrentKeyBinding1, CurrentKeyBinding2 = GetBindingKey("BreadCrumbs_OpenOptionsMenu");
										-- Check if the user pressed ESC to wipe the existing keybinding
										if not(value == "") then
											-- Check if there is an existing keybinding
											if (CurrentKeyBinding2) then
												-- Wipe original keybinding
												SetBinding(CurrentKeyBinding2);
											end;
											-- Setup new keybinding
											SetBinding(value, "BreadCrumbs_OpenOptionsMenu");
											-- Save new keybindings
											SaveBindings(GetCurrentBindingSet());
										-- If ESC was pressed to wipe the existing keybinding ensure there IS existing keybindings
										elseif (CurrentKeyBinding2) then
											-- Wipe original keybinding
											SetBinding(CurrentKeyBinding2)
											-- Save new keybindings
											SaveBindings(GetCurrentBindingSet());
										end;
									end
								},
							--------------------------------------------------------
							
							--------------------------------------------------------
							-- Warn user if there are more keybindings than are displayed
							--------------------------------------------------------
								NumOptionMenuKeybinding = {
									type = "description",
									order = 19,
									fontSize = "large",
									hidden = function()
										-- Get the number of keybindings
										local NumberOfKeybindings = TableCount({GetBindingKey("BreadCrumbs_OpenOptionsMenu")});
										-- Get if there are NOT more than 2 as a boolean
										local Hide = not(NumberOfKeybindings > 2);
										-- Return boolean
										return Hide;
									end,
									name = function()
										-- Get the number of keybindings
										local NumberOfKeybindings = TableCount({GetBindingKey("BreadCrumbs_OpenOptionsMenu")});
										-- 
										if (NumberOfKeybindings > 2) then
											-- Return a warning msg
											return ZMC_COLOURS.RED..L["WARNING! There are"]..": "..tostring(NumberOfKeybindings).." "..L["separate key-bindings for the event"].." '"..L["Options Menu"].."'!\n"..L["More than can be displayed here"].."!"..ZMC_COLOURS.CLOSE;
										else
											-- Nothing to worry about so return a blank string
											return "";
										end;
									end
								},
							--------------------------------------------------------
							
							NewLine_5 = {
								order	= 20,
								name	= "",
								type	= "description",
							},
						-----------------------------------------------------------------------------------------------
					},
				},
			--------------------------------------------------------------------------------------------------------------
			---------------------------------------------------------------------------------------------------------------
			-- Minimap's and other 3rd Party Addons
			---------------------------------------------------------------------------------------------------------------
				MinimapsAndHUDsGroup = {
					type = "group",
					name = L["Minimaps & HUDs"],
					order = 3,
					-- The args are blank currently and will be dynamicly filled in by code later
					args = {},
				},
			---------------------------------------------------------------------------------------------------------------
		},
	};
	
	-- Updates the Interface Options for the Individual Settings
	self:UpdateIndividualSettingsInterfaceOptions();
	
	------------------------------------------------------------------------------
	-- Uses the options table just created to 
	------------------------------------------------------------------------------
		-- Registers the "options" table ready to be used
		AceConfigRegistry:RegisterOptionsTable(self.Name, self.Options)
		-- Add the options group to the blizzards option menu
		self.OptionsFrame = LibStub("AceConfigDialog-3.0"):AddToBlizOptions(self.Name, self.OptionName)
        -- Also add the tabs individually encase the user likes that better
        self.OptionsFrameGeneral = LibStub("AceConfigDialog-3.0"):AddToBlizOptions(self.Name, L["General Settings"], self.OptionName, "GeneralGroup");
        self.OptionsFrameKeyBindings = LibStub("AceConfigDialog-3.0"):AddToBlizOptions(self.Name, "KeyBindings", self.OptionName, "KeyBindingsGroup");
        self.OptionsFrameMinimapsAndHUDS = LibStub("AceConfigDialog-3.0"):AddToBlizOptions(self.Name, L["Minimaps & HUDs"], self.OptionName, "MinimapsAndHUDsGroup");
	------------------------------------------------------------------------------
end;

-- Enables or Disables this addon
function BreadCrumbs:TogAddon(value)
	-------------------------------------------------
	-- ZMC Debug: If bDBug is set to true all of the
	-- debug msgs in THIS function will apear!
	-------------------------------------------------
	-- local bDBug = true; -- Uncomment this to debug
	local DBugMsg, ErrDBugMsg, DBState = InitDBug(bDBug);
	-------------------------------------------------
	
	DBugMsg("BreadCrumbs:TogAddon(value = "..tostring(value)..")");
	
	-- Store the current enabled/disabled state of the addon
	local OldEnabledState = BreadCrumbs_Options.Enabled;
	-- No value has been passed so just invert the current one
	if (value == nil) then
		if BreadCrumbs_Options.Enabled == true then
			value = false;
		else
			value = true;
		end;
	end;
	-- True passed so enable addon
	if (value == true) then
		-- Ensure the addon is currently disabled
		if (OldEnabledState == false) then
			-- Tell the addon it is now enabled! (So the functions will work...)
			BreadCrumbs_Options.Enabled = true;
			-- Start Second section of setup. Need to get a zone update yet though.
			self:InitLevel(2);
			-- Updates the options window
			AceConfigRegistry:NotifyChange(self.Name)
			-- Tell the user we ahve enabled the addon
			Msg(L["Enabled"]..".");
		else
			-- Tell the user the addon is already enabled
			ErrMsg(L["Already Enabled"].."! "..L["If you are having trouble with the addon then try reloading or disabling all other addons encase it is a conflict"]..".");
		end;
	-- False passed so enable addon
	elseif (value == false) then
		-- Tell the addon it is disabled so most of the functions won't run
		BreadCrumbs_Options.Enabled = false;
		-- Updates the options window
		AceConfigRegistry:NotifyChange(self.Name)
		-- Unloads addon when not needed
		-- self:UnloadAddon(); -- NOT DONE YET!
		-- Tell the user the addon is now disabled
		ErrMsg(L["Disabled"]..". "..L["If you are disabling this addon as you don't intend to use it for a long period you are better disabling it from the character selection screen to save memory!"]);
	else
		ErrDBugMsg("ERROR in 'TogAddon' value not specified");
	end;
end;

-- Deletes all settings and restarts UI
function BreadCrumbs:ResetAllSettings()
	-------------------------------------------------
	-- ZMC Debug: If bDBug is set to true all of the
	-- debug msgs in THIS function will apear!
	-------------------------------------------------
	-- local bDBug = true; -- Uncomment this to debug
	local DBugMsg, ErrDBugMsg, DBState = InitDBug(bDBug);
	-------------------------------------------------
	
	DBugMsg("BreadCrumbs:ResetAllSettings()");
	-- Disables BreadCrumbs to ensure it doesn't error
	self:TogAddon(false);
	-- Wipes the settings
	BreadCrumbs_Options = nil;
	-- Wipes the character specific settings
	BreadCrumbs_Char_Options = nil;
	-- Wipes the DBs
	BreadCrumbs_DebugLog = nil;
	-- Wipes the DBs
	BreadCrumbs_StoreDebugMsgs = nil;
	-- Reloads the UI so BreadCrumbs reinitialize correctly
	ReloadUI();
end;

-- Runs all the time (thousands of times a second!) and if the use has moved enough adds a new point
function BreadCrumbs:Update(frame, elapsed)
	-------------------------------------------------
	-- ZMC Debug: If bDBug is set to true all of the
	-- debug msgs in THIS function will apear!
	-------------------------------------------------
	-- local bDBug = true; -- Uncomment this to debug
	local DBugMsg, ErrDBugMsg, DBState = InitDBug(bDBug);
	-------------------------------------------------
	
	-- DBugMsg("BreadCrumbs:Update()");
	-- Check if we should continue
	if not(not(BreadCrumbs_Options.Enabled) or not(self.LoadedStatus["Initialized"] == self.LoadedStatus["RunLevel"]) or not(BreadCrumbs_Options.Dropping) or ((UnitIsDeadOrGhost("player") == 1) and BreadCrumbs_Options.DontDropWhenDead)) then
		-- update our zone info
		local curZone = GetRealZoneText();
		if (not curZone or curZone == "") then
			return;
		end;
		
		-- Calculates how long since it last updated
		self.last_update = self.last_update + elapsed;
		-- DBugMsg("self.UpdateFrame - OnUpdate");
		-- DBugMsg("elapsed = "..tostring(elapsed));
		-- DBugMsg("last_update = "..tostring(self.last_update));
		
		-- Ensures that it doesn't update to often
		if self.last_update > 0.1 then
			-- DBugMsg("BreadCrumbs:Update() - last_update > 0.5");
			
			self.last_update = 0
			
			local DistBetweenPoints = BreadCrumbs_Options.DistBetweenPoints;
			
			-----------------------------------------------------------------
			-- Store the players current position on the map in the next slot
			----------------------------------------------------------------
				-- Get current player position
				local posX, posY = GetPlayerMapPosition("player");
				
				-- DBugMsg("posX = "..tostring(posX)..", posY = "..tostring(posY)..", curZone = "..tostring(curZone).." DistBetweenPoints = "..tostring(DistBetweenPoints).."");
				
				local PrevMovementHistorySlot;
				if ((BreadCrumbs_Char_Options.NextMovementHistorySlot - 1) < 1) then
					PrevMovementHistorySlot = BreadCrumbs_Options.NumPoints;
				else 
					PrevMovementHistorySlot = BreadCrumbs_Char_Options.NextMovementHistorySlot - 1;
				end;
				
				-- If position is 0, the player changed the worldmap to another zone, just keep the old values
				if ((posX == 0) and (posY == 0) and GetCurrentMapZone() == 0) then
					DBugMsg("The player changed the worldmap to another zone, don't do anything");
				-- Coords look ok
				else
					local prevX = BreadCrumbs_Char_Options.MovementHistory[PrevMovementHistorySlot]["x"] or 0;
					local prevY = BreadCrumbs_Char_Options.MovementHistory[PrevMovementHistorySlot]["y"] or 0;
					local prevMapID = BreadCrumbs_Char_Options.MovementHistory[PrevMovementHistorySlot]["MapID"] or 0;
					local prevFloorID = BreadCrumbs_Char_Options.MovementHistory[PrevMovementHistorySlot]["FloorID"] or 0;
					
					DBugMsg("prevX = "..tostring(prevX)..", prevY = "..tostring(prevY)..", prevCont = "..tostring(prevCont).."");
					
					local curPlayerX = posX;
					local curPlayerY = posY;
					local MapID, FloorID = Astrolabe:GetUnitPosition("player", true);
					local Dist;
					-- /dump DongleStub("Astrolabe-1.0-Zas"):GetUnitPosition("player", true);
					DBugMsg("curPlayerX = "..tostring(curPlayerX)..", curPlayerY = "..tostring(curPlayerY).."");
					DBugMsg("Astrolabe:ComputeDistance(MapID = "..tostring(MapID)..", nil, curPlayerX = "..tostring(curPlayerX)..", curPlayerY = "..tostring(curPlayerY)..", prevMapID = "..tostring(prevMapID)..", nil, prevX = "..tostring(prevX)..", prevY = "..tostring(prevY)..");")
					
					-- Ensure MapID was pulled back
					if (MapID and FloorID) then
						Dist = Astrolabe:ComputeDistance(MapID, FloorID, curPlayerX, curPlayerY, prevMapID, prevFloorID, prevX, prevY);
					else
						DBugMsg("MapID or FloorID = nil");
						Dist = 0;
					end;
					
					DBugMsg("Dist = "..tostring(Dist).."");
					
					-- Make sure the distance is never nil! If it is asume it is a long way (likely another planet!) so make it DistBetweenPoints + 1...
					Dist = Dist or (DistBetweenPoints + 1);
					
					if ((Dist > DistBetweenPoints)) then
						DBugMsg("Diference IS great enough! PrevMovementHistorySlot = "..tostring(PrevMovementHistorySlot)..", BreadCrumbs_Char_Options.MovementHistorySlot = "..tostring(BreadCrumbs_Char_Options.NextMovementHistorySlot));
						DBugMsg("posX = "..tostring(posX)..", posY = "..tostring(posY)..", curZone = "..tostring(curZone).."");
						
						BreadCrumbs_Char_Options.MovementHistory[BreadCrumbs_Char_Options.NextMovementHistorySlot]["x"] = posX;
						BreadCrumbs_Char_Options.MovementHistory[BreadCrumbs_Char_Options.NextMovementHistorySlot]["y"] = posY;
						BreadCrumbs_Char_Options.MovementHistory[BreadCrumbs_Char_Options.NextMovementHistorySlot]["zone"] = curZone;
						BreadCrumbs_Char_Options.MovementHistory[BreadCrumbs_Char_Options.NextMovementHistorySlot]["MapID"] = MapID;
						BreadCrumbs_Char_Options.MovementHistory[BreadCrumbs_Char_Options.NextMovementHistorySlot]["FloorID"] = FloorID;
						
						local result = Astrolabe:RemoveIconFromMinimap(self.Points[BreadCrumbs_Char_Options.NextMovementHistorySlot])
						
						-- There was an error!
						if not(result == 0) then
							DBugMsg("There was an error trying to remove an icon from the minimap as it doesn't exist. Icon # = "..tostring(BreadCrumbs_Char_Options.NextMovementHistorySlot).."| result = "..tostring(result).."");
						end;
						
						-- Catch any errors...
						if (self.Points[BreadCrumbs_Char_Options.NextMovementHistorySlot] == nil or MapID == nil or FloorID == nil or posX == nil or posY == nil) then
							ErrMsg("VALUE IS NIL THAT SHOULDN'T BE! - self.Points[BreadCrumbs_Char_Options.NextMovementHistorySlot] = "..tostring(self.Points[BreadCrumbs_Char_Options.NextMovementHistorySlot])..", MapID = "..tostring(MapID)..", FloorID = "..tostring(FloorID)..", posX = "..tostring(posX)..", posY = "..tostring(posY).."");
						end;
						
						local result = Astrolabe:PlaceIconOnMinimap(self.Points[BreadCrumbs_Char_Options.NextMovementHistorySlot], MapID, FloorID, posX, posY);
						
						-- There was an error!
						if not(result == 0) then
							DBugMsg("There was an error trying to add icon to minimap.  Icon # = "..tostring(BreadCrumbs_Char_Options.NextMovementHistorySlot).."| result = "..tostring(result).."");
						end;
						
						DBugMsg("BreadCrumbs_Char_Options.NextMovementHistorySlot = "..tostring(BreadCrumbs_Char_Options.NextMovementHistorySlot));
						
						if (BreadCrumbs_Char_Options.NextMovementHistorySlot < BreadCrumbs_Options.NumPoints) then
							BreadCrumbs_Char_Options.NextMovementHistorySlot = BreadCrumbs_Char_Options.NextMovementHistorySlot + 1;
						else
							BreadCrumbs_Char_Options.NextMovementHistorySlot = 1;
						end;
						
						DBugMsg("BreadCrumbs_Char_Options.NextMovementHistorySlot = "..tostring(BreadCrumbs_Char_Options.NextMovementHistorySlot));
						
						if (BreadCrumbs_Options.IndividualSettings[self.ActiveMinimapsName].ColourGradiant or BreadCrumbs_Options.IndividualSettings[self.ActiveMinimapsName].SizeGradiant) then
							-- Updates the Colour/Opacity and Size of all of the points
							self:UpdatePoints();
						end;
					else
						DBugMsg("Diference not great enough! Dist = "..tostring(Dist).." > DistBetweenPoints = "..tostring(DistBetweenPoints).."");
					end;
				end;
				
				DBugMsg("xDif = "..tostring(xDif));
				DBugMsg("yDif = "..tostring(yDif));
			-----------------------------------------------------------------
		end;
	end;
end;

-- Calculates the gradiant between two values
function BreadCrumbs:CalGradiant(Value1, Value2, NumPoints)
	-------------------------------------------------
	-- ZMC Debug: If bDBug is set to true all of the
	-- debug msgs in THIS function will apear!
	-------------------------------------------------
	-- local bDBug = true; -- Uncomment this to debug
	local DBugMsg, ErrDBugMsg, DBState = InitDBug(bDBug);
	-------------------------------------------------
	
	ErrDBugMsg("BreadCrumbs:CalGradiant(Value1 = "..tostring(Value1)..", Value2 = "..tostring(Value2)..", NumPoints = "..tostring(NumPoints).."))");
	
	------------------------------------
	-- Calculate Differences
	------------------------------------
		local Diff = Value1 - Value2;
	------------------------------------
	
	------------------------------------
	-- Calculate Steps
	------------------------------------
		local Step = Diff / (NumPoints - 1); -- -1 is used as we want the number of changes from the lowest value to highest NOT the number of points (To prevent /0 errors you can't have less than 2 points!)
	------------------------------------
	
	DBugMsg("Diff = "..tostring(Diff).." Step = "..tostring(Step).."");
	
	------------------------------------
	-- Clear Arrays
	------------------------------------
		local returnAry = {};
	------------------------------------
	
	------------------------------------
	-- Sets up values for first loop
	------------------------------------
		local CurentValue = Value1;
	------------------------------------
	
	------------------------------------------------
	-- Loop for every point required
	------------------------------------------------
		for i=1, NumPoints do
			returnAry[i] = CurentValue;
			
			DBugMsg("i = "..tostring(i).." CurentValue = "..tostring(CurentValue).."");
			
			if (i == (NumPoints - 1)) then -- This is the final loop so don't bother calculating it
				DBugMsg("i == (NumPoints - 1)");
				
				CurentValue = Value2;
			else
				DBugMsg("not(i == (NumPoints - 1))");
				
				CurentValue = CurentValue - Step;
			end;
		end;
	------------------------------------------------
	
	return returnAry; -- Return the array
end;

-- Updates the gradiants with any new colours/transparancys
function BreadCrumbs:UpdateGradiants()
	-------------------------------------------------
	-- ZMC Debug: If bDBug is set to true all of the
	-- debug msgs in THIS function will apear!
	-------------------------------------------------
	-- local bDBug = true; -- Uncomment this to debug
	local DBugMsg, ErrDBugMsg, DBState = InitDBug(bDBug);
	-------------------------------------------------
	
	DBugMsg("BreadCrumbs:UpdateGradiants()");
	
	------------------------------------------------------------------------------------------------------
	-- Update ColourGradiant if required
	------------------------------------------------------------------------------------------------------
		-- Colour Gradiant is enabled so calculate them
		if (BreadCrumbs_Options.IndividualSettings[self.ActiveMinimapsName].ColourGradiant) then
			-- Calculates the gradiant between two values of Red
			self.RedColourGradArray = self:CalGradiant(BreadCrumbs_Options.IndividualSettings[self.ActiveMinimapsName].Colour1.Red, BreadCrumbs_Options.IndividualSettings[self.ActiveMinimapsName].Colour2.Red, BreadCrumbs_Options.NumPoints);
			-- Calculates the gradiant between two values of Green
			self.GreenColourGradArray = self:CalGradiant(BreadCrumbs_Options.IndividualSettings[self.ActiveMinimapsName].Colour1.Green, BreadCrumbs_Options.IndividualSettings[self.ActiveMinimapsName].Colour2.Green, BreadCrumbs_Options.NumPoints);
			-- Calculates the gradiant between two values of Blue
			self.BlueColourGradArray = self:CalGradiant(BreadCrumbs_Options.IndividualSettings[self.ActiveMinimapsName].Colour1.Blue, BreadCrumbs_Options.IndividualSettings[self.ActiveMinimapsName].Colour2.Blue, BreadCrumbs_Options.NumPoints);
			-- Calculates the gradiant between two values of Opacity
			self.TextureOpacityGradArray = self:CalGradiant(BreadCrumbs_Options.IndividualSettings[self.ActiveMinimapsName].Colour1.TextureOpacity, BreadCrumbs_Options.IndividualSettings[self.ActiveMinimapsName].Colour2.TextureOpacity, BreadCrumbs_Options.NumPoints);
		end;
	------------------------------------------------------------------------------------------------------
	
	------------------------------------------------------------------------------------------------------
	-- Update SizeGradiant if required
	------------------------------------------------------------------------------------------------------
		-- Minimap Size Gradiant is enabled so calculate them
		if (BreadCrumbs_Options.IndividualSettings[BreadCrumbs.ActiveMinimapsName].SizeGradiant) then
            DBugMsg("Calculates the gradiant between two values of the Diameter");
			self.DiameterGradArray = self:CalGradiant(BreadCrumbs_Options.IndividualSettings[self.ActiveMinimapsName].PointsDiameter_1, BreadCrumbs_Options.IndividualSettings[self.ActiveMinimapsName].PointsDiameter_2, BreadCrumbs_Options.NumPoints);
		end;
	------------------------------------------------------------------------------------------------------
end;

-- Removes and then readds the points to the minimap etc...
function BreadCrumbs:ReAddPoints()
	-------------------------------------------------
	-- ZMC Debug: If bDBug is set to true all of the
	-- debug msgs in THIS function will apear!
	-------------------------------------------------
	-- local bDBug = true; -- Uncomment this to debug
	local DBugMsg, ErrDBugMsg, DBState = InitDBug(bDBug);
	-------------------------------------------------
	
	DBugMsg("BreadCrumbs:ReAddPoints()");
	
	self:GeneratePoints(false); -- Creates the frames that will be used as the points on the map
	self:UpdatePoints() -- Updates the Colour/Opacity and Size of all of the points
	Astrolabe:RemoveAllMinimapIcons(); -- Clear any and all points already on the minimap
	
	--------------------------------------------------------------------------------------------
	-- Loop though all points
	--------------------------------------------------------------------------------------------
		for name, value in pairs(self.Points) do
			local posX = BreadCrumbs_Char_Options.MovementHistory[name]["x"];
			local posY = BreadCrumbs_Char_Options.MovementHistory[name]["y"];
			local curZone = BreadCrumbs_Char_Options.MovementHistory[name]["zone"];
			local MapID = BreadCrumbs_Char_Options.MovementHistory[name]["MapID"];
			local FloorID = BreadCrumbs_Char_Options.MovementHistory[name]["FloorID"];
			
			DBugMsg("Loop num = "..tostring(name).." - posX = "..tostring(posX)..", posY = "..tostring(posY).."");
			
			if not((posX == 0) and (posY == 0)) then -- If this point is not 0x0 (which is not imposible but very unlikely and more likely an old instance or viewing another zone on the map) then assume it's valid and add it!
				DBugMsg("Point OK Adding...");
				
				local result = Astrolabe:PlaceIconOnMinimap(self.Points[name], MapID, FloorID, posX, posY);
				
				if (result == 0) then -- All OK
					DBugMsg("Added icon to minimap.  Icon # = "..tostring(name).."| result = "..tostring(result).."");
				else -- There was an error!
					DBugMsg("There was an error trying to add icon to minimap.  Icon # = "..tostring(name).."| result = "..tostring(result).."");
				end;
			end;
		end;
	--------------------------------------------------------------------------------------------
end;

-- Creates the frames that will be used as the points on the map
function BreadCrumbs:GeneratePoints(bWipePoints)
	-------------------------------------------------
	-- ZMC Debug: If bDBug is set to true all of the
	-- debug msgs in THIS function will apear!
	-------------------------------------------------
	-- local bDBug = true; -- Uncomment this to debug
	local DBugMsg, ErrDBugMsg, DBState = InitDBug(bDBug);
	-------------------------------------------------
	
	DBugMsg("BreadCrumbs:GeneratePoints()");
	
	-- Removes all minimap icons
	Astrolabe:RemoveAllMinimapIcons();
	
	if not(bWipePoints == false) then
		DBugMsg("bWipePoints = "..tostring(bWipePoints).."");
		-- Removes all history
		self:WipePoints();
	end;
	
	-- Wipe and initialize the array to store the points
	self.Points = {};
	-- Updates the gradiants with any new colours/transparancys
	self:UpdateGradiants();
	
	--------------------------------------------------------------------------------------------
	-- Generate the specified number of points and populate them with current value sets
	--------------------------------------------------------------------------------------------
		for i=1, BreadCrumbs_Options.NumPoints do -- Loop for every point required
			---------------------------------------------------------------
			-- Set colours, Opacity, Width and Height etc... for this point
			---------------------------------------------------------------
				local Red, Green, Blue, TextureOpacity, Width, Height
				-- Colour Gradiant is enabled so use it
				if (BreadCrumbs_Options.IndividualSettings[self.ActiveMinimapsName].ColourGradiant) then
					Red = self.RedColourGradArray[i];
					Green = self.GreenColourGradArray[i];
					Blue = self.BlueColourGradArray[i];
					TextureOpacity = self.TextureOpacityGradArray[i];
				-- Colour Gradiant is disabled so use first points values
				else
					Red = BreadCrumbs_Options.IndividualSettings[self.ActiveMinimapsName].Colour1.Red;
					Green = BreadCrumbs_Options.IndividualSettings[self.ActiveMinimapsName].Colour1.Green;
					Blue = BreadCrumbs_Options.IndividualSettings[self.ActiveMinimapsName].Colour1.Blue;
					TextureOpacity = BreadCrumbs_Options.IndividualSettings[self.ActiveMinimapsName].Colour1.TextureOpacity;
				end;
				-- Size Gradiant is enabled so use it
				if (BreadCrumbs_Options.IndividualSettings[self.ActiveMinimapsName].SizeGradiant) then
					Diameter = self.DiameterGradArray[i];
				-- Size Gradiant is disabled so user first point values
				else
					Diameter = BreadCrumbs_Options.IndividualSettings[self.ActiveMinimapsName].PointsDiameter_1;
				end;
			---------------------------------------------------------------
			local Point;
			-- Check if we already have a frame for this point
			if (self.Points[i] and (type(self.Points[i]) == "Frame")) then
				-- Reuse the frame that we already have for this point on the minimap
				Point = self.Points[i];
			else
				-- Create the frame that will be the point on the minimap
				Point = CreateFrame("Frame", self.PointsName, minimapParent);
			end;
			-- Sets the frame level (this says if its on front of or behind other frames)
			Point:SetFrameLevel(Minimap:GetFrameLevel());
			-- Sets frame's Width (will likely be changed later in the code)
			Point:SetWidth(Diameter);
			-- Sets frame's Height (will likely be changed later in the code)
			Point:SetHeight(Diameter);
			-- Creates a textrue for the frame
			local texture = Point:CreateTexture(nil, "OVERLAY");
			-- Attaches the texture to the frame
			Point.texture = texture;
			-- This makes the texture stretch across all of the frame
			texture:SetAllPoints(Point);
			-- Sets the texture(image file) of the texture...
			texture:SetTexture(BreadCrumbs_Options.Texture);
			-- Sets the colour (Red, Green, Blue, TextureOpacity)
			texture:SetVertexColor(Red, Green, Blue, TextureOpacity);
			-- Hides the frame until it is needed
			Point:Hide();
			-- Store the local point into the point's table
			self.Points[i] = Point
		end;
	--------------------------------------------------------------------------------------------
end;

-- Runs when the SexyMapHUD is shown
function BreadCrumbs:HudMapCluster_OnShow()
	-------------------------------------------------
	-- ZMC Debug: If bDBug is set to true all of the
	-- debug msgs in THIS function will apear!
	-------------------------------------------------
	-- local bDBug = true; -- Uncomment this to debug
	local DBugMsg, ErrDBugMsg, DBState = InitDBug(bDBug);
	-------------------------------------------------
	
	DBugMsg("BreadCrumbs:HudMapCluster_OnShow()");
	
	if (not(BreadCrumbs_Options.Enabled)) then -- Addon is disabled so exit
		return;
	end;
	
	self:ReparentMinimap(HudMapCluster, "SexyMap")
end;

-- Runs when the SexyMapHUD is hidden
function BreadCrumbs:HudMapCluster_OnHide()
	-------------------------------------------------
	-- ZMC Debug: If bDBug is set to true all of the
	-- debug msgs in THIS function will apear!
	-------------------------------------------------
	-- local bDBug = true; -- Uncomment this to debug
	local DBugMsg, ErrDBugMsg, DBState = InitDBug(bDBug);
	-------------------------------------------------
	
	DBugMsg("BreadCrumbs:HudMapCluster_OnHide()");
	
	if (not(BreadCrumbs_Options.Enabled)) then -- Addon is disabled so exit
		return;
	end;
	
	self:ReparentMinimap(Minimap, "Minimap")
end;

-- Moves the trail to another minimap Frame
function BreadCrumbs:ReparentMinimap(minimap, CallingAddonName, CallingAddonDetails)
	-------------------------------------------------
	-- ZMC Debug: If bDBug is set to true all of the
	-- debug msgs in THIS function will apear!
	-------------------------------------------------
	-- local bDBug = true; -- Uncomment this to debug
	local DBugMsg, ErrDBugMsg, DBState = InitDBug(bDBug);
	-------------------------------------------------
	
	DBugMsg("BreadCrumbs:ReparentMinimap(minimap = "..tostring(minimap)..", CallingAddonName = "..tostring(CallingAddonName)..", CallingAddonDetails = "..tostring(CallingAddonDetails)..")");
	
	-- Check the addon is enabled
	if (BreadCrumbs_Options and BreadCrumbs_Options.Enabled) then
		DBugMsg("Check the addon has finished loading");
		-- Check the addon has finished loading
		if (self.LoadedStatus["Initialized"] == self.LoadedStatus["RunLevel"]) then
			DBugMsg("See if the addon passed it's name");
			-- See if the addon passed it's name
			if (CallingAddonName) then
				DBugMsg("See if the addon passed it's name or a table that might contain it's name");
				-- See if the addon passed it's name or a table that might contain it's name
				if (type(CallingAddonName, "string")) then
					DBugMsg("Set CallingAddonsName to the addon's name passed");
					-- Set CallingAddonsName to the addon's name passed
					CallingAddonsName = CallingAddonName;
				-- Maybe it's the addon's table and THAT contains a name...
				elseif (type(CallingAddonName, "table") and CallingAddonName.Name and type(CallingAddonName.Name, "string")) then
					DBugMsg("Set CallingAddonsName to the addon's name passed");
					-- Set CallingAddonsName to the addon's name passed
					CallingAddonsName = CallingAddonName.Name;
				-- Don't understand what was passed
				else
					DBugMsg("We couldn't work out who the addon said it was so use AnyUnknownAddons' settings");
					-- We couldn't work out who the addon said it was so use AnyUnknownAddons' settings
					CallingAddonsName = "AnyUnknownAddons";
				end;
				
				-- See if we got an addon name
				if (CallingAddonsName) then
					DBugMsg("The addon doesn't already have settings");
					-- The addon doesn't already have settings
					if not(BreadCrumbs_Options.IndividualSettings[CallingAddonsName]) then
						DBugMsg("Create some default for 3rd party addons");
						-- Create some default for 3rd party addons
						self:CreateDefaultThirdPartyAddonsettings(CallingAddonsName, CallingAddonDetails);
					end;
					
					-- Say this addon is active
					self.ActiveMinimapsName = CallingAddonsName;
				else
					DBugMsg("We couldn't work out who the addon said it was so use AnyUnknownAddons' settings");
					-- We couldn't work out who the addon said it was so use AnyUnknownAddons' settings
					CallingAddonsName = "AnyUnknownAddons";
				end;
			else
				DBugMsg("The addon didn't say who it was so use AnyUnknownAddons' settings");
				-- The addon didn't say who it was so use AnyUnknownAddons' settings
				CallingAddonsName = "AnyUnknownAddons";
			end;
			
			-- Go though all of the waypoints setting there parent to the new minimap
			for pointNum, waypoint in pairs(self.Points) do
				waypoint:SetParent(minimap)
			end
			
			-- Check if the minimap that has been passed is the Blizzard default one
			if (minimap == Minimap) then
				DBugMsg("To Minimap");
				CallingAddonsName = "Minimap";
			end
			-- Switching to Minimap so set ActiveMinimapName to that
			self.ActiveMinimapsName = CallingAddonsName;
			
			minimapParent = minimap
			Astrolabe:ReparentMinimap(minimap);
			Astrolabe.processingFrame:SetParent(minimap)
			
			DBugMsg("CallingAddonsName = "..tostring(CallingAddonsName).."");
			DBugMsg("BreadCrumbs_Options.IndividualSettings[CallingAddonsName].Hidden = "..tostring(BreadCrumbs_Options.IndividualSettings[CallingAddonsName].Hidden).."");
			
			-- Update visability
			local CurrentHiddenState;
			if (BreadCrumbs_Options.IndividualSettings[CallingAddonsName].Hidden == nil) then
				CurrentHiddenState = BreadCrumbs_Options.Hidden;
			else
				CurrentHiddenState = BreadCrumbs_Options.IndividualSettings[CallingAddonsName].Hidden;
			end;
			
			-- Removes all minimap icons and readds them
			self:RefreshPoints();
			-- Update hidden state
			self:TogHideCrumbs(CurrentHiddenState);
		-- The addon hasn't finished loading yet.
		else
			-- Store the target minimap frame for when it has.
			self.ReparentToMinimap = minimap;
			self.ReparentToCallingAddonName = CallingAddonName
			self.ReparentToCallingAddonDetails = CallingAddonDetails
		end;
	-- The addon hasn't finished loading yet.
	else
		-- Store the target minimap frame for when it has.
		self.ReparentToMinimap = minimap;
		self.ReparentToCallingAddonName = CallingAddonName
		self.ReparentToCallingAddonDetails = CallingAddonDetails
	end;
end

-- Updates the Colour/Opacity and Size of all of the points
function BreadCrumbs:UpdatePoints()
	-------------------------------------------------
	-- ZMC Debug: If bDBug is set to true all of the
	-- debug msgs in THIS function will apear!
	-------------------------------------------------
	-- local bDBug = true; -- Uncomment this to debug
	local DBugMsg, ErrDBugMsg, DBState = InitDBug(bDBug);
	-------------------------------------------------
	
	DBugMsg("BreadCrumbs:UpdatePoints()");
	
	-- Initialize MinimapInUse if it hasn't been passed in
	MinimapInUse = MinimapInUse or MinimapCluster:IsShown();
	
	-- DBugMsg("MinimapInUse = "..tostring(MinimapInUse).."");
	
	--------------------------------------------------------------------------------------------
	-- Loop though all points
	--------------------------------------------------------------------------------------------
		for Point, value in pairs(self.Points) do
			-- DBugMsg("CurrentPoint = "..tostring(Point).." BreadCrumbs_Char_Options.NextMovementHistorySlot = "..tostring(BreadCrumbs_Char_Options.NextMovementHistorySlot).."");
			
			local CurrentSlot = (BreadCrumbs_Char_Options.NextMovementHistorySlot - 1)
			if (CurrentSlot < 1) then
				CurrentSlot = CurrentSlot + BreadCrumbs_Options.NumPoints
			end;
			
			numGradPoint = (CurrentSlot - (Point - 1))
			-- DBugMsg("Point = "..tostring(Point).." numGradPoint = "..tostring(numGradPoint).." CurrentSlot = "..tostring(CurrentSlot).."");
			if (numGradPoint < 1) then
				-- DBugMsg("numGradPoint < 1");
				numGradPoint = numGradPoint + BreadCrumbs_Options.NumPoints
			end;
			
			-- DBugMsg("CurrentPoint = "..tostring(Point).." numGradPoint = "..tostring(numGradPoint).."");
			-- DBugMsg("TableCount = "..tostring(TableCount(self.Points)));
			---------------------------------------------------------------
			-- Set colours, Opacity, Width and Height etc... for this point
			---------------------------------------------------------------
				local Red, Green, Blue, TextureOpacity, Width, Height
				-- Minimap Gradiant is enabled so use it
				if (BreadCrumbs_Options.IndividualSettings[self.ActiveMinimapsName].ColourGradiant) then
					Red = self.RedColourGradArray[numGradPoint];
					Green = self.GreenColourGradArray[numGradPoint];
					Blue = self.BlueColourGradArray[numGradPoint];
					TextureOpacity = self.TextureOpacityGradArray[numGradPoint];
				-- Minimap Gradiant is disabled so use first points values
				else
					Red = BreadCrumbs_Options.IndividualSettings[self.ActiveMinimapsName].Colour1.Red;
					Green = BreadCrumbs_Options.IndividualSettings[self.ActiveMinimapsName].Colour1.Green;
					Blue = BreadCrumbs_Options.IndividualSettings[self.ActiveMinimapsName].Colour1.Blue;
					TextureOpacity = BreadCrumbs_Options.IndividualSettings[self.ActiveMinimapsName].Colour1.TextureOpacity;
				end;
				-- Size Gradiant is enabled so use it
				if (BreadCrumbs_Options.IndividualSettings[BreadCrumbs.ActiveMinimapsName].SizeGradiant) then
					DBugMsg("Size Gradiant is enabled so use it. numGradPoint = '"..tostring(numGradPoint).."'");
					Diameter = BreadCrumbs.DiameterGradArray[numGradPoint];
				-- Size Gradiant is disabled so use the first points values
				else
					-- DBugMsg("Size Gradiant is disabled so use the first points values");
					Diameter = BreadCrumbs_Options.IndividualSettings[BreadCrumbs.ActiveMinimapsName].PointsDiameter_1;
				end;
			---------------------------------------------------------------
			
			-- Initialize the should hide variable
			local bShouldHide = false;
			-- If this is within the last specified number to hide then make it transparent
			if (self.CurrentlyHidden or (numGradPoint < BreadCrumbs_Options.IndividualSettings[self.ActiveMinimapsName].LastNumToHide)) then
				-- DBugMsg("MAKE INVISIBLE: self.CurrentlyHidden = "..tostring(self.CurrentlyHidden).." or (numGradPoint = "..tostring(numGradPoint).." < BreadCrumbs_Options.IndividualSettings[self.ActiveMinimapsName].LastNumToHide = "..tostring(BreadCrumbs_Options.IndividualSettings[self.ActiveMinimapsName].LastNumToHide)..") CurrentPoint = "..tostring(Point).."");
				-- Make transparent
				bShouldHide = true;
			else
				-- DBugMsg("LEAVE IT! - (numGradPoint = "..tostring(numGradPoint)..") < (BreadCrumbs_Options.IndividualSettings[self.ActiveMinimapsName = "..tostring(self.ActiveMinimapsName).."].LastNumToHide( = "..tostring(BreadCrumbs_Options.IndividualSettings[self.ActiveMinimapsName].LastNumToHide)..") + 1)");
			end;
			
			-- DBugMsg("TextureOpacity = "..tostring(TextureOpacity)..", Red = "..tostring(Red)..", Green = "..tostring(Green)..", Blue = "..tostring(Blue)..", Diameter = "..tostring(Diameter).."");
			
			---------------------------------------------------------------
			-- Store the new value for this point
			---------------------------------------------------------------
                -- DBugMsg("Diameter = '"..tostring(Diameter).."'");
				-- Sets frame's Width (will likely be changed later in the code)
				value:SetWidth(Diameter);
				-- Sets frame's Height (will likely be changed later in the code)
				value:SetHeight(Diameter);
				-- Sets the colour (Red, Green, Blue, TextureOpacity)
				value.texture:SetVertexColor(Red, Green, Blue, TextureOpacity);
				-- Set the visability of the new point
				if bShouldHide then
					value:Hide();
                    -- DBugMsg("Hidding Point = '"..tostring(value:IsVisible()).."'");
				else
					value:Show();
                    -- DBugMsg("Showing Point = '"..tostring(value:IsVisible()).."'");
				end;
			---------------------------------------------------------------
		end;
	--------------------------------------------------------------------------------------------
end;

-- Removes all minimap icons and readds them
function BreadCrumbs:RefreshPoints()
	-------------------------------------------------
	-- ZMC Debug: If bDBug is set to true all of the
	-- debug msgs in THIS function will apear!
	-------------------------------------------------
	-- local bDBug = true; -- Uncomment this to debug
	local DBugMsg, ErrDBugMsg, DBState = InitDBug(bDBug);
	-------------------------------------------------
	
	DBugMsg("BreadCrumbs:RefreshPoints()");
	
	-- Loop though all points hideing them
	for Key, Value in pairs (self.Points) do
		Value:Hide();
	end;
	
	--------------------------------------------------------------------------------------------
	-- Loop though all points
	--------------------------------------------------------------------------------------------
		for name, value in pairs(self.Points) do
			local posX = BreadCrumbs_Char_Options.MovementHistory[name]["x"];
			local posY = BreadCrumbs_Char_Options.MovementHistory[name]["y"];
			local curZone = BreadCrumbs_Char_Options.MovementHistory[name]["zone"];
			local MapID = BreadCrumbs_Char_Options.MovementHistory[name]["MapID"];
			local FloorID = BreadCrumbs_Char_Options.MovementHistory[name]["FloorID"];
			
			DBugMsg("Loop num = "..tostring(name).." - posX = "..tostring(posX)..", posY = "..tostring(posY).."");
			
			-- If this point is not 0x0 (which is not imposible but very unlikely and more likely an old instance or viewing another zone on the map) then assume it's valid and add it!
            if not((posX == 0) and (posY == 0)) then
				DBugMsg("Point OK Adding...");
				
				local result = Astrolabe:PlaceIconOnMinimap(self.Points[name], MapID, FloorID, posX, posY);
				
				if (result == 0) then -- All OK
					DBugMsg("Added icon to minimap.  Icon # = "..tostring(name).."| result = "..tostring(result).."");
				else -- There was an error!
					DBugMsg("There was an error trying to add icon to minimap.  Icon # = "..tostring(name).."| result = "..tostring(result).."");
				end;
			end;
		end;
	--------------------------------------------------------------------------------------------
    
    -- Updates the gradiants with any new colours/transparancys
    self:UpdateGradiants();
    -- Updates the Colour/Opacity and Size of all of the points
    self:UpdatePoints();
end;

-- Removes all minimap icons
function BreadCrumbs:WipePoints()
	-------------------------------------------------
	-- ZMC Debug: If bDBug is set to true all of the
	-- debug msgs in THIS function will apear!
	-------------------------------------------------
	-- local bDBug = true; -- Uncomment this to debug
	local DBugMsg, ErrDBugMsg, DBState = InitDBug(bDBug);
	-------------------------------------------------
	
	DBugMsg("BreadCrumbs:WipePoints()");
	-- Stores the next slot number for the position
	BreadCrumbs_Char_Options.NextMovementHistorySlot = 1
	-- Wipe and resetup the movement history
	BreadCrumbs_Char_Options.MovementHistory = {};
	for i=1, BreadCrumbs_Options.NumPoints do
		BreadCrumbs_Char_Options.MovementHistory[i] = {}
		BreadCrumbs_Char_Options.MovementHistory[i]["x"] = 0
		BreadCrumbs_Char_Options.MovementHistory[i]["y"] = 0
	end;
	-- Removes all minimap icons from Astrolabe and hide them
	Astrolabe:RemoveAllMinimapIcons();
	-- If points already exist then move them to a junk frame
	if (self.Points and TableCount(self.Points) > 0) then
		-- Create (if required) a junk frame
		if not(self.JunkFrame) then
			self.JunkFrame = CreateFrame("Frame", self.Name.."_JunkFrame", UIParent);
		end;
		-- Move all of the points onto a junk frame for now
		for Key, Value in pairs(self.Points) do
			Value:SetParent(self.JunkFrame);
		end
	end;
end;

-- Starts/Stops dropping a trail. (This won't delete or hide the existing trail)
function BreadCrumbs:TogBreadDrop(value)
	-------------------------------------------------
	-- ZMC Debug: If bDBug is set to true all of the
	-- debug msgs in THIS function will apear!
	-------------------------------------------------
	-- local bDBug = true; -- Uncomment this to debug
	local DBugMsg, ErrDBugMsg, DBState = InitDBug(bDBug);
	-------------------------------------------------
	
	DBugMsg("BreadCrumbs:TogBreadDrop(value = "..tostring(value)..")");
	
	if (value == nil) then -- No value given so just toggle it to the opposite of it's current state
		if (BreadCrumbs_Options.Dropping) then
			value = false;
		else
			value = true;
		end;
	end;
	
	if (value) then -- Value specifed as true so turn it on
		BreadCrumbs_Options.Dropping = true;
		
		AceConfigRegistry:NotifyChange(self.Name) -- Updates the options window
		
		Msg(L["Started dropping BreadCrumbs"]);
	else -- Value specified and it's not true so assume it's false and turn it off
		BreadCrumbs_Options.Dropping = false;
		
		AceConfigRegistry:NotifyChange(self.Name) -- Updates the options window
		
		ErrMsg(L["Stopped dropping BreadCrumbs"]);
	end;
end;

-- Turns Gradiant on/off for the points
function BreadCrumbs:TogColourGrad(value, AddonName)
	-------------------------------------------------
	-- ZMC Debug: If bDBug is set to true all of the
	-- debug msgs in THIS function will apear!
	-------------------------------------------------
	-- local bDBug = true; -- Uncomment this to debug
	local DBugMsg, ErrDBugMsg, DBState = InitDBug(bDBug);
	-------------------------------------------------
	
	DBugMsg("BreadCrumbs:TogColourGrad(value = "..tostring(value)..")");
	
	-- No value given so just toggle it to the opposite of it's current state
	if (value == nil) then
		if (BreadCrumbs_Options.IndividualSettings[AddonName].ColourGradiant) then
			value = false;
		else
			value = true;
		end;
	end;
	-- Value specifed as true so turn it on
	if (value) then
		-- Only do anything if the tog has changed
		if (not(BreadCrumbs_Options.IndividualSettings[AddonName].ColourGradiant)) then
			DBugMsg("BreadCrumbs_Options.IndividualSettings[AddonName].ColourGradiant = true;");
			BreadCrumbs_Options.IndividualSettings[AddonName].ColourGradiant = true;
			-- Updates the options window
			AceConfigRegistry:NotifyChange(self.Name)
			-- Updates the gradiants with any new colours/transparancys
			self:UpdateGradiants();
			-- Updates the Colour/Opacity and Size of all of the points
			self:UpdatePoints();
			
			Msg(L["Enabled"].." "..tostring(AddonName).." "..L["Gradient"]);
		else
			DBugMsg("value = "..tostring(value).." BreadCrumbs_Options.IndividualSettings[AddonName].ColourGradiant = "..tostring(BreadCrumbs_Options.IndividualSettings[AddonName].ColourGradiant).."");
		end;
	-- Value specified and it's not true so assume it's false and turn it off
	else
		-- Only do anything if the tog has changed
		if (BreadCrumbs_Options.IndividualSettings[AddonName].ColourGradiant) then
			DBugMsg("BreadCrumbs_Options.IndividualSettings[AddonName].ColourGradiant = false;");
			BreadCrumbs_Options.IndividualSettings[AddonName].ColourGradiant = false;
			-- Updates the options window
			AceConfigRegistry:NotifyChange(self.Name);
			-- Updates the Colour/Opacity and Size of all of the points
			self:UpdatePoints();
			
			ErrMsg(L["Disabled"].." "..tostring(AddonName).." "..L["Gradient"]);
		else
			DBugMsg("value = "..tostring(value).." BreadCrumbs_Options.IndividualSettings[AddonName].ColourGradiant = "..tostring(BreadCrumbs_Options.IndividualSettings[AddonName].ColourGradiant).."");
		end;
	end;
end;

-- Turns Size Gradiant on/off for the Minimap points
function BreadCrumbs:TogSizeGrad(value, AddonName)
	-------------------------------------------------
	-- ZMC Debug: If bDBug is set to true all of the
	-- debug msgs in THIS function will apear!
	-------------------------------------------------
	-- local bDBug = true; -- Uncomment this to debug
	local DBugMsg, ErrDBugMsg, DBState = InitDBug(bDBug);
	-------------------------------------------------
	
	DBugMsg("BreadCrumbs:TogSizeGrad(value = "..tostring(value)..")");
	-- No value given so just toggle it to the opposite of it's current state
	if (value == nil) then
		if (BreadCrumbs_Options.IndividualSettings[AddonName].SizeGradiant) then
			value = false;
		else
			value = true;
		end;
	end;
	-- Value specifed as true so turn it on
	if (value) then
		-- Only do anything if the tog has changed
		if (not(BreadCrumbs_Options.IndividualSettings[AddonName].SizeGradiant)) then
			BreadCrumbs_Options.IndividualSettings[AddonName].SizeGradiant = true;
			-- Updates the options window
			AceConfigRegistry:NotifyChange(self.Name);
			-- Updates the gradiants with any new colours/transparancys
			self:UpdateGradiants();
			-- Updates the Colour/Opacity and Size of all of the points
			self:UpdatePoints();
			
			Msg(L["Enabled"].." "..tostring(AddonName).." "..L["Size Gradient"]);
		else
			DBugMsg("value = "..tostring(value).." BreadCrumbs_Options.IndividualSettings[AddonName].SizeGradiant = "..tostring(BreadCrumbs_Options.IndividualSettings[AddonName].SizeGradiant).."");
		end;
	-- Value specified and it's not true so assume it's false and turn it off
	else
		DBugMsg("BreadCrumbs_Options.IndividualSettings[AddonName = "..tostring(AddonName).."] = "..tostring(BreadCrumbs_Options.IndividualSettings[AddonName]).."");
		-- Only do anything if the tog has changed
		if (BreadCrumbs_Options.IndividualSettings[AddonName].SizeGradiant) then
			BreadCrumbs_Options.IndividualSettings[AddonName].SizeGradiant = false;
			-- Updates the options window
			AceConfigRegistry:NotifyChange(self.Name);
			-- Updates the Colour/Opacity and Size of all of the points
			self:UpdatePoints();
			
			ErrMsg(L["Disabled"].." "..tostring(AddonName).." "..L["Size Gradient"]);
		else
			DBugMsg("value = "..tostring(value).." BreadCrumbs_Options.IndividualSettings[AddonName].SizeGradiant = "..tostring(BreadCrumbs_Options.IndividualSettings[AddonName].SizeGradiant).."");
		end;
	end;
end;

-- Hides/unhides the trail. (This won't delete the trail and it will still be making one in the background unless it's stopped)
function BreadCrumbs:TogHideCrumbs(value)
	-------------------------------------------------
	-- ZMC Debug: If bDBug is set to true all of the
	-- debug msgs in THIS function will apear!
	-------------------------------------------------
	-- local bDBug = true; -- Uncomment this to debug
	local DBugMsg, ErrDBugMsg, DBState = InitDBug(bDBug);
	-------------------------------------------------
	
	DBugMsg("BreadCrumbs:TogHideCrumbs(value = "..tostring(value)..")");
	
	-- Initialize Value Passed boolean
	local bValuePassed = false;
	
	-- No value given so just toggle it to the opposite of it's current state
	if (value == nil) then
		DBugMsg("value == nil");
		
		if (self.CurrentlyHidden) then
			value = false;
		else
			value = true;
		end;
	else
		bValuePassed = true;
	end;
	
	-- Value specifed as true so turn it on
	if (value) then
		DBugMsg("value == "..tostring(value));
		
		self.CurrentlyHidden = true;
		-- Updates the options window
		AceConfigRegistry:NotifyChange(self.Name)
		-- Only display message it if it's hidden...
		if not(self.CurrentlyHidden) then	
			ErrMsg(L["Hidden"]);
		end;
	-- Value specified and it's not true so assume it's false and turn it off
	else
		DBugMsg("value == "..tostring(value));
		
		self.CurrentlyHidden = false;
		-- Updates the options window
		AceConfigRegistry:NotifyChange(self.Name)
		-- Only display message if it's visable...
		if (self.CurrentlyHidden) then	
			Msg(L["Unhidden"]);
		end;
	end;
	
	-- Updates the Colour/Opacity and Size of all of the points
	self:UpdatePoints();
	
	-- Store the new state in the correct setting
	if (BreadCrumbs_Options.IndividualSettings[self.ActiveMinimapsName].Hidden == nil) then
		BreadCrumbs_Options.Hidden = self.CurrentlyHidden;
		if not(bValuePassed) then
			Msg("Crumb visability hidden state set to: '"..tostring(self.CurrentlyHidden).."'");
		end;
	else
		BreadCrumbs_Options.IndividualSettings[self.ActiveMinimapsName].Hidden = self.CurrentlyHidden
		if not(bValuePassed) then
			Msg(tostring(self.ActiveMinimapsName).." visability override set to: '"..tostring(self.CurrentlyHidden).."'");
		end;
	end;
end;

-- Stops/Starts dropping a trail when you are dead. (This won't delete or hide the existing trail)
function BreadCrumbs:TogDropWhenDead(value)
	-------------------------------------------------
	-- ZMC Debug: If bDBug is set to true all of the
	-- debug msgs in THIS function will apear!
	-------------------------------------------------
	-- local bDBug = true; -- Uncomment this to debug
	local DBugMsg, ErrDBugMsg, DBState = InitDBug(bDBug);
	-------------------------------------------------
	
	DBugMsg("BreadCrumbs:TogDropWhenDead(value = "..tostring(value)..")");
	
	if (value == nil) then -- No value given so just toggle it to the opposite of it's current state
		DBugMsg("value == nil");
		
		if (BreadCrumbs_Options.DontDropWhenDead) then
			value = false;
		else
			value = true;
		end;
	end;
	
	if (value) then -- Value specifed as true so turn it on
		DBugMsg("value == "..tostring(value));
		
		if (BreadCrumbs_Options.DontDropWhenDead == false) then -- Only change if needed... (lets you call this function knowing it won't do anything if not required)
			BreadCrumbs_Options.DontDropWhenDead = true;
			
			AceConfigRegistry:NotifyChange(self.Name) -- Updates the options window
			
			Msg(L["Don't Drop When Dead Enabled"]);
		end;
	else -- Value specified and it's not true so assume it's false and turn it off
		DBugMsg("value == "..tostring(value));
		
		if (BreadCrumbs_Options.DontDropWhenDead == true) then -- Only hide it if it's visable... (lets you call this to hide it knowing it won't do anything if not required)
			BreadCrumbs_Options.DontDropWhenDead = false;
			
			AceConfigRegistry:NotifyChange(self.Name) -- Updates the options window
			
			ErrMsg(L["Don't Drop When Dead Disabled"]);
		end;
	end;
end;

-- Ensures that there are options and an option page for specified addon
function BreadCrumbs:CreateDefaultThirdPartyAddonsettings(CallingAddonsName, CallingAddonsDetails)
	-------------------------------------------------
	-- ZMC Debug: If bDBug is set to true all of the
	-- debug msgs in THIS function will apear!
	-------------------------------------------------
	-- local bDBug = true; -- Uncomment this to debug
	local DBugMsg, ErrDBugMsg, DBState = InitDBug(bDBug);
	-------------------------------------------------
	
	DBugMsg("BreadCrumbs:CreateDefaultThirdPartyAddonsettings(CallingAddonsName = "..tostring(CallingAddonsName)..")");
	
	-- If this addon's details haven't already been stored and where provided by the addon then store them
	if (BreadCrumbs_Options.IndividualSettings[CallingAddonsName] == nil) then
		DBugMsg("No details stored so see if they were provided");
		-- No details stored so see if they were provided
		if (type(CallingAddonsDetails) == "table") then
			DBugMsg("Details were provided so store them");
			-- Details were provided so store them
			BreadCrumbs_Options.IndividualSettings[CallingAddonsName] = CopyTable(CallingAddonsDetails);
		end;
	end;
	
	-- Initializes Minimap & Addon's Individual Settings Defaults
	self:InitIndividualSetting(CallingAddonsName);
	-- Updates the Interface Options for the Individual Settings
	self:UpdateIndividualSettingsInterfaceOptions();
	-- Update the Interface Options with the changes we just made
	AceConfigRegistry:NotifyChange(self.Name);
end;

-- Updates (or creates) the Interface Options for the Individual Settings
function BreadCrumbs:UpdateIndividualSettingsInterfaceOptions()
	-------------------------------------------------
	-- ZMC Debug: If bDBug is set to true all of the
	-- debug msgs in THIS function will apear!
	-------------------------------------------------
	-- local bDBug = true; -- Uncomment this to debug
	local DBugMsg, ErrDBugMsg, DBState = InitDBug(bDBug);
	-------------------------------------------------
	
	DBugMsg("BreadCrumbs:UpdateIndividualSettingsInterfaceOptions()");
	
	-- Ensure the MinimapsAndHUDsGroup's "args" are emptyWipe old options to recreate them.
	self.Options.args.MinimapsAndHUDsGroup.args = {};
	
	-- Initialize variable to store what 
	local iOptionNum = iOptionNum or 0;
	-- Loop though all of the Minimaps and 3rd party addon's we know about and add settings for them
	for AddonName, value in pairs(BreadCrumbs_Options.IndividualSettings) do
		-- increase iOptionNum by one
		iOptionNum = iOptionNum + 1;
		
		-- create the options for this 3rd party addon/minimap
		self.Options.args.MinimapsAndHUDsGroup.args[AddonName] = {
			type = "group",
			name = AddonName,
			order = iOptionNum,
			args = {
				---------------------------------------------------------------------------------------------------
				-- First Point's Colour/Opacity for the Points
				---------------------------------------------------------------------------------------------------
					ColourHeader = {
						order	= 1,
						type	= "header",
						name	= L["Crumb's"].." "..L["Colour/Opacity"],
					},
					-- Creates and Sets Up the Last Point's Colour/Opacity for the Points
					Colour2 = {
						type = "color",
						name = L["Last Crumb colour to"].." -->",
						desc = L["This sets the Colour & Opacity level of the last crumb that make up the BreadCrumbs when on"].." "..tostring(AddonName),
						order = 2,
						hasAlpha = true,
						get = function(info)
							local Red = BreadCrumbs_Options.IndividualSettings[AddonName].Colour2.Red;
							local Green = BreadCrumbs_Options.IndividualSettings[AddonName].Colour2.Green;
							local Blue = BreadCrumbs_Options.IndividualSettings[AddonName].Colour2.Blue;
							local TextureOpacity = BreadCrumbs_Options.IndividualSettings[AddonName].Colour2.TextureOpacity;
							
							return Red, Green, Blue, TextureOpacity;
						end,
						set = function(info, Red, Green, Blue, TextureOpacity)
							DBugMsg("Colour2 - setFunc(Red = "..tostring(Red)..", Green = "..tostring(Green)..", Blue = "..tostring(Blue)..", TextureOpacity = "..tostring(TextureOpacity)..")");
							BreadCrumbs_Options.IndividualSettings[AddonName].Colour2.Red = Red;
							BreadCrumbs_Options.IndividualSettings[AddonName].Colour2.Green = Green;
							BreadCrumbs_Options.IndividualSettings[AddonName].Colour2.Blue = Blue;
							BreadCrumbs_Options.IndividualSettings[AddonName].Colour2.TextureOpacity = TextureOpacity;
							-- Updates the gradiants with any new colours/transparancys
							self:UpdateGradiants();
							-- Updates the Colour/Opacity and Size of all of the points
							self:UpdatePoints();
						end,
						hidden = function()
							DBugMsg("return not(BreadCrumbs_Options.IndividualSettings[AddonName = "..tostring(AddonName).."].ColourGradiant("..tostring(BreadCrumbs_Options.IndividualSettings[AddonName].ColourGradiant).."));");
							return not(BreadCrumbs_Options.IndividualSettings[AddonName].ColourGradiant);
						end,
					},
					-- Creates and Sets Up the First Point's Colour/Opacity for the Minimap
					Colour1 = {
						type = "color",
						name = L["Crumb"].." "..L["Colour/Opacity"],
						desc = L["This sets the Colour & Opacity level of the first crumb (or all if gradient is disabled) that make up the BreadCrumbs when on the Minimap."],
						order = 3,
						hasAlpha = true,
						get = function(info)
							local Red = BreadCrumbs_Options.IndividualSettings[AddonName].Colour1.Red;
							local Green = BreadCrumbs_Options.IndividualSettings[AddonName].Colour1.Green;
							local Blue = BreadCrumbs_Options.IndividualSettings[AddonName].Colour1.Blue;
							local TextureOpacity = BreadCrumbs_Options.IndividualSettings[AddonName].Colour1.TextureOpacity;
							
							return Red, Green, Blue, TextureOpacity;
						end,
						set = function(info, Red, Green, Blue, TextureOpacity)
							DBugMsg("Colour1 - setFunc");
							BreadCrumbs_Options.IndividualSettings[AddonName].Colour1.Red = Red;
							BreadCrumbs_Options.IndividualSettings[AddonName].Colour1.Green = Green;
							BreadCrumbs_Options.IndividualSettings[AddonName].Colour1.Blue = Blue;
							BreadCrumbs_Options.IndividualSettings[AddonName].Colour1.TextureOpacity = TextureOpacity;
							-- Updates the gradiants with any new colours/transparancys
							self:UpdateGradiants();
							-- Updates the Colour/Opacity and Size of all of the points
							self:UpdatePoints();
						end,
					},
					-- Creates and Sets Up the Gradiant Toggle
					Gradiant = {
						type = "toggle",
						name = L["Gradient"],
						desc = L["Gradually change the colour/transparency between the first and last point on the Minimap"],
						order = 4,
						get = function(info)
							return BreadCrumbs_Options.IndividualSettings[AddonName].ColourGradiant;
						end,
						set = function(info, value)
							self:TogColourGrad(value, AddonName);
						end,
						width = "full",
					},
				---------------------------------------------------------------------------------------------------
				
				---------------------------------------------------------------------------------------------------
				-- Size Settings
				---------------------------------------------------------------------------------------------------
					SizeHeader = {
						order	= 5,
						type	= "header",
						name	= L["Crumb's Sizes"],
					},
					-- Creates and Sets Up the SizeOfPoints2 slider
					SizeOfPoints2 = {
						type = "range",
						name = L["Last Point Size to"].." -->",
						desc = L["This slider specifies the size of the first point on the minimap is."],
						order = 6,
						min = 1,
						max = 50,
						step = 1,
						get = function(info)
							return BreadCrumbs_Options.IndividualSettings[AddonName].PointsDiameter_2;
						end,
						set = function(info, value)
							BreadCrumbs_Options.IndividualSettings[AddonName].PointsDiameter_2 = value;
							BreadCrumbs_Options.IndividualSettings[AddonName].PointsDiameter_2 = value;
							-- Updates the gradiants with any new colours/transparancys
							self:UpdateGradiants();
							self:UpdatePoints();
						end,
						hidden = function()
							DBugMsg("return not(BreadCrumbs_Options.IndividualSettings[AddonName].SizeGradiant("..tostring(BreadCrumbs_Options.IndividualSettings[AddonName].SizeGradiant).."));");
							return not(BreadCrumbs_Options.IndividualSettings[AddonName].SizeGradiant);
						end,
					},
					-- Creates and Sets Up the SizeOfPoints1 slider
					SizeOfPoints1 = {
						type = "range",
						name = L["Crumb Size"],
						desc = L["This slider specifies how big the last point on the minimap is."],
						order = 7,
						min = 1,
						max = 50,
						step = 1,
						get = function(info)
							return BreadCrumbs_Options.IndividualSettings[AddonName].PointsDiameter_1;
						end,
						set = function(info, value)
							BreadCrumbs_Options.IndividualSettings[AddonName].PointsDiameter_1 = value;
							BreadCrumbs_Options.IndividualSettings[AddonName].PointsDiameter_1 = value;
							-- Updates the gradiants with any new colours/transparancys
							self:UpdateGradiants();
							self:UpdatePoints();
						end,
					},
					-- Creates and Sets Up the Size Gradiant Toggle
					SizeGradiant = {
						type = "toggle",
						name = L["Gradient"],
						desc = L["Gradually change the size between the first and last point on the Minimap"],
						order = 8,
						get = function(info)
							return BreadCrumbs_Options.IndividualSettings[AddonName].SizeGradiant;
						end,
						set = function(info, value)
							self:TogSizeGrad(value, AddonName);
						end,
						width = "full",
					},
				---------------------------------------------------------------------------------------------------
				
				---------------------------------------------------------------------------------------------------
				-- Hidden Settings
				---------------------------------------------------------------------------------------------------
					HiddenHeader = {
						order	= 9,
						type	= "header",
						name	= L["Visibility Override"],
					},
					-- Visability Override toggle
					VisOverrideHeader = {
						order	= 10,
						type	= "select",
						name	= L["Visibility Override"],
						style = "radio",
						values = {
							ZZ_DISABLE =  ZMC_COLOURS.DKRED..L["No Override"]..ZMC_COLOURS.CLOSE,
							On = L["Always Hide"],
							Off = L["Never Hide"],
						},
						get = function()
							local returnVal = BreadCrumbs_Options.IndividualSettings[AddonName].Hidden;
							if (returnVal == nil) then
								returnVal = "ZZ_DISABLE";
							elseif (returnVal == true) then
								returnVal = "On";
							elseif (returnVal == false) then
								returnVal = "Off";
							end;
							return returnVal;
						end,
						set = function(info, value)
							if (value == "ZZ_DISABLE") then
								value = nil;
							elseif (value == "On") then
								value = true;
							elseif (value == "Off") then
								value = false;
							end;
							BreadCrumbs_Options.IndividualSettings[AddonName].Hidden = value;
							-- Update the points
							local currentHiddenState;
							if (BreadCrumbs_Options.IndividualSettings[self.ActiveMinimapsName].Hidden == nil) then
								currentHiddenState = BreadCrumbs_Options.Hidden;
							else
								currentHiddenState = BreadCrumbs_Options.IndividualSettings[self.ActiveMinimapsName].Hidden;
							end
							self:TogHideCrumbs(currentHiddenState);
						end
					},
					-- Creates and Sets Up the LastNumToHide slider
					LastNumToHide = {
						type = "range",
						name = L["Hide Closest # Points"],
						desc = L["This slider specifies how many of the closest crumbs to the player (if any) should be hidden."].."\n\n"..L["This is to help prevent the players arrow from being covered."],
						order = 11,
						min = 1,
						max = 50,
						step = 1,
						get = function(info)
							return BreadCrumbs_Options.IndividualSettings[AddonName].LastNumToHide;
						end,
						set = function(info, value)
							BreadCrumbs_Options.IndividualSettings[AddonName].LastNumToHide = value;
							-- Updates the gradiants with any new colours/transparancys
							self:UpdateGradiants();
							self:UpdatePoints();
						end,
					},
				---------------------------------------------------------------------------------------------------
			},
		};
	end;
end;

-- Initializes Minimap & Addon's Individual Settings Defaults
function BreadCrumbs:InitIndividualSetting(AddonName)
	-------------------------------------------------
	-- ZMC Debug: If bDBug is set to true all of the
	-- debug msgs in THIS function will apear!
	-------------------------------------------------
	-- local bDBug = true; -- Uncomment this to debug
	local DBugMsg, ErrDBugMsg, DBState = InitDBug(bDBug);
	-------------------------------------------------
	
	DBugMsg("BreadCrumbs:InitIndividualSetting(AddonName = "..tostring(AddonName)..")");
	
	BreadCrumbs_Options.IndividualSettings[AddonName] = BreadCrumbs_Options.IndividualSettings[AddonName] or {};
	
	-----------------------------------------
	-- Colour 1
	-----------------------------------------
		BreadCrumbs_Options.IndividualSettings[AddonName].Colour1 = BreadCrumbs_Options.IndividualSettings[AddonName].Colour1 or {};
		
		BreadCrumbs_Options.IndividualSettings[AddonName].Colour1.Red = BreadCrumbs_Options.IndividualSettings[AddonName].Colour1.Red or 1;
		
		BreadCrumbs_Options.IndividualSettings[AddonName].Colour1.Green = BreadCrumbs_Options.IndividualSettings[AddonName].Colour1.Green or 0;
		
		BreadCrumbs_Options.IndividualSettings[AddonName].Colour1.Blue = BreadCrumbs_Options.IndividualSettings[AddonName].Colour1.Blue or 0;
		
		BreadCrumbs_Options.IndividualSettings[AddonName].Colour1.TextureOpacity = BreadCrumbs_Options.IndividualSettings[AddonName].Colour1.TextureOpacity or 0.78;
	-----------------------------------------
	
	-----------------------------------------
	-- Colour 2
	-----------------------------------------
		BreadCrumbs_Options.IndividualSettings[AddonName].Colour2 = BreadCrumbs_Options.IndividualSettings[AddonName].Colour2 or {};
		
		BreadCrumbs_Options.IndividualSettings[AddonName].Colour2.Red = BreadCrumbs_Options.IndividualSettings[AddonName].Colour2.Red or 0;
		
		BreadCrumbs_Options.IndividualSettings[AddonName].Colour2.Green = BreadCrumbs_Options.IndividualSettings[AddonName].Colour2.Green or 1;
		
		BreadCrumbs_Options.IndividualSettings[AddonName].Colour2.Blue = BreadCrumbs_Options.IndividualSettings[AddonName].Colour2.Blue or 0;
		
		BreadCrumbs_Options.IndividualSettings[AddonName].Colour2.TextureOpacity = BreadCrumbs_Options.IndividualSettings[AddonName].Colour2.TextureOpacity or 0.78;
	-----------------------------------------
	
	-----------------------------------------
	-- Diameter - 1 & 2
	-----------------------------------------
		BreadCrumbs_Options.IndividualSettings[AddonName].PointsDiameter_1 = BreadCrumbs_Options.IndividualSettings[AddonName].PointsDiameter_1 or 4;
		
		BreadCrumbs_Options.IndividualSettings[AddonName].PointsDiameter_2 = BreadCrumbs_Options.IndividualSettings[AddonName].PointsDiameter_2 or 4;
	-----------------------------------------
	
	
	-----------------------------------------
	-- Gradiants
	-----------------------------------------
		if (BreadCrumbs_Options.IndividualSettings[AddonName].ColourGradiant == nil) then
			BreadCrumbs_Options.IndividualSettings[AddonName].ColourGradiant = true;
		end;
		
		if (BreadCrumbs_Options.IndividualSettings[AddonName].SizeGradiant == nil) then
			BreadCrumbs_Options.IndividualSettings[AddonName].SizeGradiant = false;
		end;
	-----------------------------------------
	
	
	-----------------------------------------
	-- Others
	-----------------------------------------
		BreadCrumbs_Options.IndividualSettings[AddonName].LastNumToHide = BreadCrumbs_Options.IndividualSettings[AddonName].LastNumToHide or 3;
	-----------------------------------------
end;

-- Updates/sets the number of points to use and updates everything
function BreadCrumbs:UpdateNumberOfPoints(newValue)
	-------------------------------------------------
	-- ZMC Debug: If bDBug is set to true all of the
	-- debug msgs in THIS function will apear!
	-------------------------------------------------
	-- local bDBug = true; -- Uncomment this to debug
	local DBugMsg, ErrDBugMsg, DBState = InitDBug(bDBug);
	-------------------------------------------------
	
	DBugMsg("BreadCrumbs:UpdateNumberOfPoints(newValue = "..tostring(newValue)..")");
	
	-- Ensure newValue is a number
	newValue = tonumber(newValue)
	-- If no change then don't do anything!
	if (BreadCrumbs_Options.NumPoints == newValue) then
		DBugMsg("BreadCrumbs_Options.NumPoints == newValue");
		return;
	end;
	-- Set the number of points to the ones specified
	BreadCrumbs_Options.NumPoints = newValue;
	-- Recreate the new number of points
	self:GeneratePoints();
	
	if (HudMapCluster) then
		DBugMsg("HudMapCluster Exists!");
		
		if (HudMapCluster:IsShown()) then
			DBugMsg("and HudMapCluster is Shown!");
			-- Hide the SexyMap's HUD
			HudMapCluster:Hide();
			-- Now Show the SexyMap's HUD again to reset everything.
			HudMapCluster:Show();
		else
			DBugMsg("and HudMapCluster is NOT Shown!");
		end;
	else
		DBugMsg("HudMapCluster Does NOT Exist!");
	end;
end;

-- Ensures that our point name is added to Minimap Button Frames exception list
function BreadCrumbs:AddToMBFException()
	-------------------------------------------------
	-- ZMC Debug: If bDBug is set to true all of the
	-- debug msgs in THIS function will apear!
	-------------------------------------------------
	-- local bDBug = true; -- Uncomment this to debug
	local DBugMsg, ErrDBugMsg, DBState = InitDBug(bDBug);
	-------------------------------------------------
	
	DBugMsg("BreadCrumbs:AddToMBFException()");
	
	-- Ensure that MBF exists and that the Minimap Icons list exits
	if (MBF and MBF.db and MBF.db.profile and MBF.db.profile.MinimapIcons) then
		-- Check that BreadCrumbsPoints has not already been added to MBF
		for Key, Value in pairs (MBF.db.profile.MinimapIcons) do
			-- Is this our point name?
			if (Value == self.PointsName) then
				-- Our point name already exists so exit function
				return
			end;
		end;
		-- Point name doesn't already exist so add it.
		tinsert(MBF.db.profile.MinimapIcons, self.PointsName);
		-- Ensure that the ClearMBF() function exists
		if (ClearMBF) then
			-- Tell MBF to clear it's points
			ClearMBF();
		end
	end;
end;

-- Ensures that our point name is removed from Minimap Button Frames button override list
function BreadCrumbs:RemoveFromMBFOverride()
	-------------------------------------------------
	-- ZMC Debug: If bDBug is set to true all of the
	-- debug msgs in THIS function will apear!
	-------------------------------------------------
	-- local bDBug = true; -- Uncomment this to debug
	local DBugMsg, ErrDBugMsg, DBState = InitDBug(bDBug);
	-------------------------------------------------
	
	DBugMsg("BreadCrumbs:RemoveFromMBFOverride()");
	
	-- Ensure that MBF exists and that the ButtonOverride list exits
	if (MBF and MBF.db and MBF.db.profile and MBF.db.profile.ButtonOverride) then
        DBugMsg("ButtonOverride list exits");
        -- Generate copy of ButtonOverride
        local ButtonOverride = {};
        local iCurLoop = 0;
        -- Check that BreadCrumbsPoints has NOT been added to ButtonOverride and if it has remove it!
		for Key, Value in pairs (MBF.db.profile.ButtonOverride) do
            DBugMsg("Key = "..tostring(Key).." Value = "..tostring(Value).."");
			-- Is this our point name?
			if not(Value == self.PointsName) then
                DBugMsg("This is NOT our point so let it stay...");
                iCurLoop = iCurLoop + 1;
                MBF.db.profile.ButtonOverride[iCurLoop] = Value;
            else
                DBugMsg("This is our point so ignore it");
			end;
		end;
        -- Check if we removed any...
        if (iCurLoop < TableCount(MBF.db.profile.ButtonOverride)) then
            DBugMsg("We removed items so loop though the extra at the end wiping them");
            -- Loop though remaining values nulling them
            for i = (iCurLoop + 1), TableCount(MBF.db.profile.ButtonOverride) do
                DBugMsg("Wipe this entry as it isn't needed! i = "..tostring(i).."");
                MBF.db.profile.ButtonOverride[i] = nil;
            end;
        end;
        -- Ensure that the ClearMBF() function exists
        if (ClearMBF) then
            -- Tell MBF to clear it's points
            ClearMBF();
        end;
    end;
end;

-- Sets/Toggles between the visability overwride options for current Minimap/3rd party addon
function BreadCrumbs:TogHideCrumbsOverwride(value)
	-------------------------------------------------
	-- ZMC Debug: If bDBug is set to true all of the
	-- debug msgs in THIS function will apear!
	-------------------------------------------------
	-- local bDBug = true; -- Uncomment this to debug
	local DBugMsg, ErrDBugMsg, DBState = InitDBug(bDBug);
	-------------------------------------------------
	
	DBugMsg("BreadCrumbs:TogHideCrumbsOverwride(value = "..tostring(value)..")");
	
	-- Check what was passed
	if (value == nil) then
		-- No value given so just toggle it to the opposite of it's current state
		DBugMsg("value == nil");
		
		if (BreadCrumbs_Options.IndividualSettings[self.ActiveMinimapsName].Hidden == nil) then
			BreadCrumbs_Options.IndividualSettings[self.ActiveMinimapsName].Hidden = not(self.CurrentlyHidden);
			Msg(tostring(self.ActiveMinimapsName).." visability override enabled and set to: '"..tostring(BreadCrumbs_Options.IndividualSettings[self.ActiveMinimapsName].Hidden).."'");
		else
			BreadCrumbs_Options.IndividualSettings[self.ActiveMinimapsName].Hidden = nil;
			Msg(tostring(self.ActiveMinimapsName).." visability override disabled");
		end;
	elseif (value == true) then
		BreadCrumbs_Options.IndividualSettings[self.ActiveMinimapsName].Hidden = true;
		Msg(tostring(self.ActiveMinimapsName).." visability override set to: 'true'");
	elseif (value == false) then
		BreadCrumbs_Options.IndividualSettings[self.ActiveMinimapsName].Hidden = false;
		Msg(tostring(self.ActiveMinimapsName).." visability override set to: 'true'");
	elseif (value == "off") then
		BreadCrumbs_Options.IndividualSettings[self.ActiveMinimapsName].Hidden = nil;
		Msg(tostring(self.ActiveMinimapsName).." visability override disabled");
	end;
	
	-- Work out what the visability is now
	value = BreadCrumbs_Options.IndividualSettings[self.ActiveMinimapsName].Hidden or BreadCrumbs_Options.Hidden;
	
	if (value) then -- Value specifed as true so turn it on
		DBugMsg("value == "..tostring(value));
		
		self.CurrentlyHidden = true;
		-- Updates the options window
		AceConfigRegistry:NotifyChange(self.Name);
		-- Only display message it if it's hidden...
		if not(self.CurrentlyHidden) then	
			ErrMsg(L["Hidden"]);
		end;
	else -- Value specified and it's not true so assume it's false and turn it off
		DBugMsg("value == "..tostring(value));
		
		self.CurrentlyHidden = false;
		-- Updates the options window
		AceConfigRegistry:NotifyChange(self.Name)
		-- Only display message if it's visable...
		if (self.CurrentlyHidden) then	
			Msg(L["Unhidden"]);
		end;
	end;
	
	-- Updates the Colour/Opacity and Size of all of the points
	self:UpdatePoints();
	
	-- Store the new state in the correct setting
	if (BreadCrumbs_Options.IndividualSettings[self.ActiveMinimapsName].Hidden == nil) then
		BreadCrumbs_Options.Hidden = self.CurrentlyHidden;
	else
		BreadCrumbs_Options.IndividualSettings[self.ActiveMinimapsName].Hidden = self.CurrentlyHidden
	end;
end;


--#-#-#-#-#-#-#-#-#-#-#-#-#-#-#-#-#-#-#-#-#-#-#-#-#-#-#-#-#--
--------------------- NOT COMPLETE YET! ---------------------
--#-#-#-#-#-#-#-#-#-#-#-#-#-#-#-#-#-#-#-#-#-#-#-#-#-#-#-#-#--

-- Store error reports and display them to the user
function BreadCrumbs:ErrorReport(sError, sErrorSeverity)
	-------------------------------------------------
	-- ZMC Debug: If bDBug is set to true all of the
	-- debug msgs in THIS function will apear!
	-------------------------------------------------
	-- local bDBug = true; -- Uncomment this to debug
	local DBugMsg, ErrDBugMsg, DBState = InitDBug(bDBug);
	-------------------------------------------------
	
	DBugMsg("BreadCrumbs:ErrorReport(sError = "..tostring(sError)..", sErrorSeverity = "..tostring(sErrorSeverity)..")");
	
	-- If it is really bad tell the user to report it (along with any other errors)
	if (sErrorSeverity == "ErrorNow") then
		
	-- Else if this error is not that bad then store it and don't bother the user yet
	else
		
	end;
end;


--#-#-#-#-#-#-#-#-#-#-#-#-#-#-#-#-#-#-#-#-#-#-#-#-#-#-#-#-#--
--------------------- NOT COMPLETE YET! ---------------------
--#-#-#-#-#-#-#-#-#-#-#-#-#-#-#-#-#-#-#-#-#-#-#-#-#-#-#-#-#--

-- Unloads addon when not needed
--[[function BreadCrumbs:UnloadAddon() -- TODO: ERR All of it!
	-------------------------------------------------
	-- ZMC Debug: If bDBug is set to true all of the
	-- debug msgs in THIS function will apear!
	-------------------------------------------------
	-- local bDBug = true; -- Uncomment this to debug
	local DBugMsg, ErrDBugMsg, DBState = InitDBug(bDBug);
	-------------------------------------------------
	
	DBugMsg("BreadCrumbs:UnloadAddon()");
	
	-- 
	
	-- Say we are now initialized level 1 (as we are going backwards as we unload!)
	self.LoadedStatus["Initialized"] = 1;
end;--]]

-- Catch when this addon has finished loading
BreadCrumbs:RegisterEvent("ADDON_LOADED");
BreadCrumbs:SetScript("OnEvent", BreadCrumbs.OnEvent);